// Control.cpp Copyright 1999 Steve Ludtke
// 1/14/99   This is where most of the logic for 'boxer' resides
#include "Control.h"

#define Inherited ControlData

#include <cmath>
#include <qprogressdialog.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qutil.h>
#include <qkeycode.h>
#include "OneLiner.h"
#include "AutoboxD.h"
#include "AutoboxSNR.h"
#include "AutomaskD.h"
#include "DriftmaskD.h"

#ifdef UBUNTU
	#include <qapplication.h>
	#include <qfiledialog.h>
#else	//UBUNTU
	#include <qapp.h>
	#include <qfiledlg.h>
#endif	//UBUNTU

namespace {
	const unsigned int BUF_SIZE = 1024;
}

//#define DEBUG0

Control::Control(QWidget* parent,const char* name):Inherited( parent, name ) {
	QPopupMenu *m1;

	setCaption( "Boxer" );
	
	mgrp->setExclusive(TRUE);

	// FILE MENU
	m1=new QPopupMenu(NULL,"File");
	menu->insertItem(QString("File"),m1,100);
	m1->insertItem(QString("Read Micrograph"),this,SLOT(Mreadimage()),0);
	m1->insertItem(QString("Save Micrograph"),this,SLOT(Msaveimage()),0);
	m1->insertItem(QString("Save Micrograph (8 bit)"),this,SLOT(Msaveimage8()),0);
	m1->insertSeparator();
	m1->insertItem(QString("Exit"),this,SLOT(Mquit()),CTRL+Key_Q);

	// BOXES MENU
	m1=new QPopupMenu(NULL,"Boxes");
	menu->insertItem(QString("Boxes"),m1);
	m1->insertItem(QString("Save Boxed Particles"),this,SLOT(Msaveclips()),CTRL+SHIFT+Key_S);
	m1->insertSeparator();
	m1->insertItem(QString("Read Box DB from Ptcl"),this,SLOT(Mreadbdbimg()),0);
	m1->insertItem(QString("Read Box DB"),this,SLOT(Mreadbdb()),0);
	m1->insertItem(QString("Save Box DB"),this,SLOT(Msavebdb()),CTRL+Key_S);
	m1->insertItem(QString("Adjust Boxes"),this,SLOT(adjustBoxes()),0);
	m1->insertItem(QString("Focal Pair Autoalign"),this,SLOT(Mfocalauto()),0);
	m1->insertSeparator();
	m1->insertItem(QString("Autobox"),this,SLOT(autoBox1()),0);
	m1->insertItem(QString("Fast Autobox"),this,SLOT(autoBoxS()),0);
	m1->insertItem(QString("Autobox from references"),this,SLOT(autoBoxM()),0);
	m1->insertItem(QString("Autobox from references (with SNR)"),this,SLOT(autoBoxSNR()),0);
	m1->insertItem(QString("Autobox from SVD"),this,SLOT(autoBoxSVD()),0);
	m1->insertItem(QString("Clear Boxes"),this,SLOT(delAll()),0);
	m1->insertItem(QString("Automask"),this,SLOT(Mautomask()),0);
	m1->insertItem(QString("Undo Automask"),this,SLOT(Munautomask()),0);
	m1->insertItem(QString("Grid box"),this,SLOT(Mgridbox()),0);
	m1->insertItem(QString("Zero -> Noise"),this,SLOT(Mzerotonoise()),0);
	m1->insertItem(QString("Resize boxes"),this,SLOT(Mnewboxsize()),0);
	m1->insertItem(QString("Iterative centering"),this,SLOT(Mcenalignint()));
	
	// PROCESS MENU
	m1=new QPopupMenu(NULL,"PROCESS");
	menu->insertItem(QString("Process"),m1);
	m1->insertItem(QString("Normalize"),this,SLOT(Mnormalize()),0);
	m1->insertItem(QString("Stripe Filter"),this,SLOT(Mstripefilter()),0);
	m1->insertItem(QString("Wiener Filter"),this,SLOT(Mwienerfilter()),0);
	m1->insertItem(QString("Check for drift"),this,SLOT(Mcheckdrift()),0);
	m1->insertItem(QString("Drift-based masking"),this,SLOT(Mdriftmask()),0);
//	m1->insertItem(QString("SVD Basis Filter"),this,SLOT(Msvdfilt()),0);
	m1->insertItem(QString("Median Filter (3x3)"),this,SLOT(Mmedian3()),0);
	m1->insertItem(QString("Median Filter (5x5)"),this,SLOT(Mmedian5()),0);
	m1->insertItem(QString("Median Filter (7x7)"),this,SLOT(Mmedian7()),0);
	m1->insertItem(QString("Median Filter (9x9)"),this,SLOT(Mmedian9()),0);

	menu->insertSeparator();

	// HELP MENU
	m1=new QPopupMenu(NULL,"Help");
	menu->insertItem(QString("Help"),m1);
	m1->insertItem("Help",this,SLOT(Mhelp()),0);
	m1->insertItem(QString("About boxer"),this,SLOT(Mabout()),0);
	

	imview=new Image;
	imview->resize(500,500);
	imview->setScale(1.0);
	imview->show();
	
	clipview=new ImageMx();
	clipview->setMode(-1);
	clipview->resize(520,300);
	clipview->viewport()->setPalette(QPalette(black));
	clipview->show();

// This restores the window geometries
	FILE *in = in=DFopenr("boxerwin");
	if (in) {
		int x,y,w,h;
#ifndef SGI		
		if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) move(x,y);
		if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) clipview->setGeometry(x,y,w,h);
		if (fscanf(in," %d %d %d %d",&x,&y,&w,&h)==4) imview->setGeometry(x,y,w,h);
#endif
		fclose(in);
	}

	ptcl.setAutoDelete(TRUE);
	abptcl.setAutoDelete(TRUE);
	abvec.setAutoDelete(TRUE);

	scale=1.0;
	apix=4.6667;
	boxsize=64;
	olap=10;
	data=NULL;
	amd=NULL;
	dmd=NULL;
	bestmap=NULL;

	curshape=-1;
	
	connect(measure,SIGNAL(clicked()),this,SLOT(setMeasure()));
	connect(helix,SIGNAL(clicked()),this,SLOT(setHelix()));
	connect(select,SIGNAL(clicked()),this,SLOT(setSelect()));
	connect(zap,SIGNAL(clicked()),this,SLOT(setDelete()));
	connect(draw,SIGNAL(clicked()),this,SLOT(setDraw()));

	connect(clipview,SIGNAL(mousePress(QMouseEvent *,int ,int , int)),this,
		SLOT(clipClick(QMouseEvent *,int ,int , int)));

	select->toggle();
        setSelect();

	connect(Size,SIGNAL(textChanged(const QString&)),this,SLOT(setSize(const QString&)));
	connect(APix,SIGNAL(textChanged(const QString&)),this,SLOT(setAPix(const QString&)));
	connect(Olap,SIGNAL(textChanged(const QString&)),this,SLOT(setOLap(const QString&)));
	connect(scalesel,SIGNAL(textChanged(const QString&)),this,SLOT(setScale(const QString&)));

//	connect(brite,SIGNAL(valueChanged(int)),this,SLOT(setBC(int)));
//	connect(cont,SIGNAL(valueChanged(int)),this,SLOT(setBC(int)));

	connect(imview,SIGNAL(mousePress(QMouseEvent *)),this,SLOT(imDown(QMouseEvent *)));
	connect(imview,SIGNAL(mouseMove(QMouseEvent *)),this,SLOT(imMove(QMouseEvent *)));
	connect(imview,SIGNAL(mouseRelease(QMouseEvent *)),this,SLOT(imUp(QMouseEvent *)));
	connect(imview,SIGNAL(resized(int,int)),SLOT(imResized(int,int)));
	connect(imview,SIGNAL(moved(int,int)),SLOT(imMoved(int,int)));

	connect(pan,SIGNAL(newValue(float,float)),SLOT(panned(float,float)));
	
	connect(helixmode,SIGNAL(activated(int)),SLOT(updHelix(int)));
}


Control::~Control() {
delete data;
delete imview;
delete clipview;
}

Image* Control::getImageView() { return imview; }
void Control::minimizeClipView() { clipview->showMinimized(); }

void Control::Mhelp() {
	char s[BUF_SIZE];
	string sc = "QHelp " + string(EMDIR()) + "/doc/progs/boxer.html&";
	if(sc.size() > BUF_SIZE-1) {
//		printf("You path %s is too long!!!!!", sc);
		std::cerr << "Your path " << sc << "is too long!!!" << std::endl;
		exit(-1);
	}
	sprintf(s,"QHelp %s/doc/progs/boxer.html&", EMDIR());
	run(s);
}

void Control::panned(float x,float y) {
imview->setOrigin(x,y);
}

void Control::setMeasure() {
mode=MEASURE;
}

void Control::setHelix() {
mode=HELIX;
}

void Control::setSelect() {
mode=SELECT;
}

void Control::setDelete() {
mode=REMOVE;
}

void Control::setDraw() {
mode=DRAW;
}

void Control::setSize(const QString &t) {
boxsize=t.toInt();
}

void Control::setAPix(const QString &t) {
apix=t.toFloat();
}

void Control::setOLap(const QString &t) {
olap=t.toFloat();
}

void Control::setScale(const QString &t) {
scale=t.toFloat();
imview->setScale(scale);
imResized(imview->width(),imview->height());
}

void Control::setBC(int value) {
/*float b=(brite->value()-500.0)/500.0;
float c=cont->value()/1000.0;
Image *im;

imview->setBrite(b);
imview->setCont(c);

clipview->setBC(b,c);*/

}

void Control::Mabout() {
	QMessageBox::about(this,"About Boxer",
 "boxer\nCopyright 1999 Steve Ludtke\n\n\
 Boxer is designed to cut out particle\n\
 images from whole micrographs. It has\n\
 features for both manual and automatic\n\
 particle selection.\n");
}

void Control::readimage(char *fsp,int num,int denom,int bsize) {
int x0,y0,w,h;

if (num<0) {
	QApplication::setOverrideCursor( waitCursor );

	printf("Reading Image: %s\n",fsp);

	if (data) delete data;
	data=new EMData;
	data->setProgCB(doQprog);
	data->readImage(fsp,0);
//	data->normalize();
//	if (data->Mean()>1.0||data->Sigma()>10.0) data->normalize();
	QFileInfo file(fsp);
	mgraph_id=file.baseName();
	imview->setData(data);
        setAPix(QString().setNum(data->Pixel()));
	imResized(imview->width(),imview->height());


	QApplication::restoreOverrideCursor( );
	return;
}

if (num>=denom) { printf("num must be less than denom\n"); exit(1); }

if (sqrt((float)denom)*sqrt((float)denom)!=(float)denom) { printf("denom must be 4,9,16,25,etc.\n"); exit(1); }
denom=(int)floor(sqrt((float)denom));

data=new EMData;
data->setProgCB(doQprog);
if (data->readMRC(fsp,1)!=0) {
	printf("The 'area' argument only works with MRC images\n"); exit(1); 
}

QApplication::setOverrideCursor( waitCursor );

w=data->xSize()/denom;
h=data->ySize()/denom;
x0=(num%denom)*w;
y0=(num/denom)*h;
w+=bsize;
h+=bsize;
if (x0+w>=data->xSize()) w=data->xSize()-x0-1;
if (y0+h>=data->ySize()) h=data->ySize()-y0-1;

delete data;

data=new EMData;
data->setProgCB(doQprog);
printf("Reading area %d,%d %d,%d\n",x0,y0,w,h);
data->readMRCArea(fsp,x0,y0,w,h);
data->normalize();
imview->setData(data);
setAPix(QString().setNum(data->Pixel()));
imResized(imview->width(),imview->height());
QApplication::restoreOverrideCursor( );

}

// Checks each particle for drift
void Control::Mcheckdrift() { 
int i,j,k,ct=0;
int x0,y0,xx,yy,nx,ny;
ImShape *ss;
EMData *av,*f;
EMData rf[16];
float *d,a,r,x,y,v0,v1,aa[16];
QList<EMData> avs;

QProgressDialog progress( "Drift detection", 0, ptcl.count(), this, 0 ,TRUE );
progress.setProgress( 0 );

avs.setAutoDelete(FALSE);
nx=ptcl.at(0)->xSize()+2;
ny=ptcl.at(1)->ySize();

// We generate 16 references in 90 degrees. The other 90 degrees is
// just a sign flip.
for (k=0; k<16; k++) {
	rf[k].setSize(nx,ny,1);	// cos
	rf[k].setComplex(1);
	rf[k].setRI(0);
	d=rf[k].getData();

	ct=0;
	for (i=0; i<nx; i+=2) {
		for (j=0; j<ny; j++) {
			a=atan2((float)j-ny/2.0,(float)i/2.0);
			r=hypot((float)j-ny/2.0,(float)i/2.0);
			if (r>=ny/2 || i==0) d[i+j*nx]=0;
			else {
				aa[k]=k*PI/16.0+PI/64.0;
				d[i+j*nx]=cos(a*2.0+aa[k])
							*exp(-(r*r*4.0)/(ny*ny))*(tanh(r/3.0-2.0)+1);
				aa[k]/=2.0;
			}

			d[i+j*nx+1]=0;
		}
	}
	rf[k].doneData();
//	rf[k].writeIMAGIC("ref.hed",-1);
}

//r1.normalize();
//r2.normalize();
//printf("%f %f %f\n",r1.dot(&r3,1),r2.dot(&r3,1),r2.dot(&r2,1));

// All the 'particles' are replaced with their FFT's
clipview->clearData();
for (i=0; i<ptcl.count(); i++) {
	EMData *tmp=ptcl.at(i)->doFFT();
	ptcl.at(i)->gimmeFFT();
	ptcl.remove(i);
	ptcl.insert(i,tmp);
	clipview->addData(tmp);
}

// Now we check each for drift
for (i=0; i<ptcl.count(); i++) {
	ss=imview->shapeAt(i);
	x0=ss->x0;
	y0=ss->y0;
	av=ptcl.at(i)->copy(0,0);
	ct=0;
	for (j=0; j<ptcl.count(); j++) {
		if (j==i) continue;
		ss=imview->shapeAt(j);
		xx=ss->x0;
		yy=ss->y0;

		if (hypot((float)xx-x0,(float)yy-y0)>2*boxsize) continue;
		f=ptcl.at(j);
		av->addIncoherent(f);
		ct++;
	}
	av->multConst(1.0/(float)ct);

	v0=1.0e10;
	v1=0;
	a=0;
	for (k=0; k<16; k++) {
		x=av->dot(&rf[k]);
		if (fabs(x)<v0) v0=fabs(x);
		if (fabs(x)>v1) {
			v1=fabs(x); 
			if (x>0) a=aa[k]; else a=aa[k]+PI/2.0; 
		}
	}
	r=800000.0*(v1-v0)/(ny*ny);

//	printf("%d.\t%4.2f\t%4.2f\t%4.2f (%1.2f)\n",i,v0,v1,r,a*180.0/PI);

	ss=imview->shapeAt(i);
	ss->type=ImShape::LINE;
	ss->lw=2;
	ss->x0=(int)floor(x0+boxsize/2-r*sin(a));
	ss->y0=(int)floor(y0+boxsize/2-r*cos(a));
	ss->x1=(int)floor(x0+boxsize/2+r*sin(a));
	ss->y1=(int)floor(y0+boxsize/2+r*cos(a));
	ss->userf=r;
	
//	av->ap2ri();
//	avs.append(av);
	delete av;
	progress.setProgress(i);
}

/*
ptcl.clear();
char s[BUF_SIZE];
for (i=0; i<avs.count(); i++) {
	sprintf(s,"AA.%d.mrc",i);
	avs.at(i)->writeMRC(s);
	ptcl.append(avs.at(i)); 
}
printf("%d new particles\n",ptcl.count());
*/
}

// This will mask out areas of an image with strong drift
void Control::Mdriftmask() {
ImShape *ss;
float max=0;
int i;

if (!dmd) dmd=new DriftmaskD();

dmd->setTarget(this);

for (i=0; i<ptcl.count(); i++) {
	ss=imview->shapeAt(i);
	if (ss->userf>max) max=ss->userf;
	int xc,yc;
	ss->type=ImShape::NONE;
	xc=(ss->x0+ss->x1)/2.0;
	yc=(ss->y0+ss->y1)/2.0;
	ss->x0=xc-floor(boxsize/1.4142);
	ss->y0=yc-floor(boxsize/1.4142);
	ss->x1=floor(boxsize*1.4142);
	ss->y1=floor(boxsize*1.4142);
	if (ss->x0<0||ss->y0<0||ss->x0+ss->x1>=data->xSize()||ss->y0+ss->y1>=data->ySize()) ss->userf=-1.0;
}

if (max<=2.0) {
	int r=QMessageBox::information(NULL,"Notice","You must issue the 'check for drift' command before\n\
using this option. It is suggested that you also use gridboxing\nrather than particle autoboxing.");
	return;
}

dmd->setThrMinMax(0,max);
dmd->show();

}

// This is the worker routine for driftmasking
void Control::driftMask(float thr,int preview) {
ImShape *ss;
int i,x,y,xx,yy;

if (preview) {
	for (i=0; i<ptcl.count(); i++) {
		ss=imview->shapeAt(i);

		if (ss->userf>thr) {
			ss->type=ImShape::FILL_CIRCLE;
		}
		else ss->type=ImShape::NONE;
	}
	imview->update();
	return;
}

float *d=data->getData();
EMData msk;
ss=imview->shapeAt(0);
msk.setSize(ss->x1,ss->y1);
msk.one();
msk.applyMask(boxsize/1.414,5);
float *d2=msk.getData();

for (i=0; i<ptcl.count(); i++) {
	ss=imview->shapeAt(i);
	if (ss->userf>thr) {
		for (x=ss->x0,xx=0; x<ss->x0+ss->x1; x++,xx++) {
			for (y=ss->y0,yy=0; y<ss->y0+ss->y1; y++,yy++) {
				d[x+y*data->xSize()]*=d2[xx+yy*(int)ss->x1];
			}
		}
		
	}
}

data->doneData();
delAll();
imview->update();
}

void Control::Mwienerfilter() {
int r=QMessageBox::information(NULL,"Error","Sorry, this option is not supported yet. CTFIT has \
this functionality.");

}

void Control::Mnormalize() {
data->normalize();
}

// This will remove vertical bands produced by linear CCD detector
// normalization problems
void Control::Mstripefilter() {
float *dat,av;
float sig=data->Sigma();
int x,y,nx,ny;

dat=data->getData();
nx=data->xSize();
ny=data->ySize();

QProgressDialog progress( "Filtering", 0, nx, this, 0 ,TRUE );
progress.setProgress( 0 );

for (x=0; x<nx; x++) {
	av=0;
	for (y=ny/4; y<3*ny/4; y++) {	// average over only the middle 1/2
		av+=dat[x+y*nx];
	}
	av/=(ny/2);
	
	for (y=0; y<ny; y++) {
		dat[x+y*nx]=(dat[x+y*nx]-av)/sig;
	}
progress.setProgress( x );
}

data->doneData();
return;
}

void Control::Mmedian3() { 
doMedian(1);
}

void Control::Mmedian5() { 
doMedian(2);
}

void Control::Mmedian7() { 
doMedian(3);
}

void Control::Mmedian9() { 
doMedian(4);
}

// This basically does the same thing as the cenalignint program
// That is, it iteratively centers the boxed particles by aligning them
// to the rotational average of all of the particles
void Control::Mcenalignint() {
EMData avg;
int i,j,maxtran,del=0;
float t,tr;

maxtran=boxsize/8;
unlink("avg.hed");
unlink("avg.img");

avg.setSize(boxsize,boxsize,1);

for (i=0; i<10; i++) {		// max iterations
	avg.zero();	 // no makeAverage for QLists
	for (j=0; j<ptcl.count(); j++) {
		avg.add(ptcl.at(j));
	}
	avg.radialAverage();
	avg.normalize();
	avg.applyMask(-1,4);
	avg.writeIMAGIC("avg.hed",-1);
	
	t=0;
	for (j=0; j<ptcl.count(); j++) {
		ImShape *s=imview->shapeAt(j);
		EMData *p = new EMData;
		p=ptcl.at(j)->copy();
		tr=p->transAlign(&avg,1,1,maxtran);
		t+=tr;

		// delete boxes that move too much in 2 sucessive iterations
		if (i>2 && tr>maxtran/2 && s->userf>maxtran/2) {
			delClip(j);
			j--;
			delete p;
			printf("%d. %f & %f  DELETE\n",j,s->userf,tr);
			del++;
			continue;
		}

		s->x0-=p->Dx();
		s->y0-=p->Dy();
		s->userf=tr;
		boxMoved(j);

		printf("%d. %f\t%f\t%f\n",j,p->Dx(),p->Dy(),tr);
		delete p;
	}
	t/=ptcl.count();
	
	printf("Iteration %d complete, mean shift = %f\n",i,t);
	if (t<1.0) break;
}

printf("%d particles deleted, leaving %d\n",del,ptcl.count());
}


void Control::doMedian(int v) {
QApplication::setOverrideCursor( waitCursor );
data->realFilter(20,v);
printf("Filter %d\n",v);
QApplication::restoreOverrideCursor( );
imview->imageChanged();
}

void Control::Mreadimage() { 
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;
QFileInfo file(s);
mgraph_id=file.baseName();

QApplication::setOverrideCursor( waitCursor );

if (data) delete data;
data=new EMData;
data->setProgCB(doQprog);
data->readImage(s.data(),0);
//if (data->Mean()>1.0||data->Sigma()>10.0) data->normalize();
imview->setData(data);
imResized(imview->width(),imview->height());

QApplication::restoreOverrideCursor( );

}

void Control::Msaveimage8() {
QString s( QFileDialog::getSaveFileName(0,"*.mrc") ),t;
if ( s.isNull() ) return;
int lx,ly;
EMData *d;
int fno=1;

QApplication::setOverrideCursor( waitCursor );

data->writeMRC(s.data(),MODE_char);

QApplication::restoreOverrideCursor( );

}

void Control::Msaveimage() {
QString s( QFileDialog::getSaveFileName(0,"*.mrc") ),t;
if ( s.isNull() ) return;
int lx,ly;
EMData *d;
int fno=1;

QApplication::setOverrideCursor( waitCursor );

data->writeMRC(s.data());

QApplication::restoreOverrideCursor( );

}

void Control::Msaveclips() {
QString defaultClipfile = mgraph_id + QString(".img");
QString s( QFileDialog::getSaveFileName(defaultClipfile,"IMAGIC format (*.img);;HDF 5 format (*.hdf *.hd5);;SPIDER format (*.spi);;SPIDER byte-swapped (*.ssw);;MRC format (*.mrc)") ),t;
if ( s.isNull() ) return;
int lx,ly;
EMData *d;
int fno=1,i=0;
int type=0;

if (s.contains(".hed")||s.contains(".img")) type=1;
else if (s.contains(".spi")) type=2;
else if (s.contains(".mrc")) type=3;
else if (s.contains(".ssw")) type=4;	// spider with swapped byte-order
//else { type=1; error(ERR_INFO,"File didn't end in .hed, .spi, .ssw or .mrc. Using IMAGIC."); }

QApplication::setOverrideCursor( waitCursor );

d=ptcl.first();
lx=d->xSize();
ly=d->ySize();

for (d=ptcl.first(),i=0; d!=NULL; d=ptcl.next(),i++) {
	if (d->xSize()!=lx || d->ySize()!=ly) {
		t.sprintf(".%d",fno++);
		if (s[int(s.length()-4)]=='.') s.insert(s.length()-4,t);
		else s.append(t);
	}
	d->setMicrographId(mgraph_id.data());
	d->set_particle_location_center_x(imview->shapeAt(i)->x0+d->xSize()/2);
	d->set_particle_location_center_y(imview->shapeAt(i)->y0+d->ySize()/2);
	d->set_center_x(d->xSize()/2);
	d->set_center_y(d->ySize()/2);
#ifdef DEBUG
	printf("micrograph id=%s\tparticle center (x,y) in micrograph=%.2f,%.2f\tparticle center (x,y) in individual box=%.2f,%.2f\n",mgraph_id.data(),d->get_particle_location_center_x(),d->get_particle_location_center_y(),d->get_center_x(),d->get_center_y());
#endif
	if (type==1) d->writeIMAGIC((const char *)s,-1);
	else if (type==2) d->writeSPIDER((const char *)s,-1);
	else if (type==4) d->writeSPIDER((const char *)s,-1,1);
	else if (type==3) d->writeMRC((const char *)(s+QString::number(i)+QString(".mrc")));
	else d->writeImage((const char *)s,-1);
	lx=d->xSize();
	ly=d->ySize();
}

QApplication::restoreOverrideCursor( );

}

void Control::Mreadbdbimg() {
int i,j=0,x0,y0,x1,y1;
char t[BUF_SIZE];
EMData *d,r;
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

int n=fileCount((const char *)s);

for (i=0; i<n; i++) {
	r.readImage((const char *)s,i,1);
	if (sscanf(r.Name()," %d,%d",&x0,&y0)!=2) {
		if (sscanf(r.Name(),"!$ %d,%s %d,%d",&j,t,&x0,&y0)!=4)  continue;
	}
	x1=r.xSize();
	y1=r.ySize();

	if (x0>data->xSize() ||x1<0 || y0>data->ySize() || y1<0) continue;
	j=newClip(x0,y0,x1,y1);
	d=data->clip(x0,y0,x1,y1);
	d->ufloat=d->Sigma();
//	d->normalize();

	if (data->hasCTFF()) {
		char st[BUF_SIZE],*t;
		string sc = string(data->Name());
		if(sc.size() > BUF_SIZE-24) {
			printf("The name %s is too long!!!", data->Name());
			exit(-1);
		}
		strcpy(st,data->Name());
		if ((t=strchr(st+3,' '))) *t=0;
		sprintf(st+strlen(st)," %d,%d",x0,y0);
		d->setCTFF(st);
	}
	else {
		char st[BUF_SIZE];
		sprintf(st,"%d,%d : %55s",x0,y0,data->Path());
		d->setName(st);
	}

	ptcl.remove(j);
	ptcl.insert(j,d);
	clipview->addData(d);
}

boxsize=x1; 
sprintf(t,"%d",x1); 
Size->setText(t);

//boxname=s;
redisplayBoxes();
}

void Control::Mreadbdb() {
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;
Mreadbdb(s.data());
}

void Control::Mreadbdb(const char* file) {
int i,j=0,x0,y0,x1,y1,u;
char t[80];
EMData *d;
FILE *in=fopen(file,"rb");
if (!in) return;

for (i=0; fgets(t,79,in); i++) {
	u=-3;
	if (sscanf(t," %d %d %d %d %d",&x0,&y0,&x1,&y1,&u)<4) break;
	if (x0>data->xSize() ||x1<0 || y0>data->ySize() || y1<0) continue;
	j=newClip(x0,y0,x1,y1);
	ImShape *s1=imview->shapeAt(j);
	if (s1) {
		s1->user2=u;
		if (u>=0) s1->color=magenta;
	}
	d=data->clip(x0,y0,x1,y1);
	d->ufloat=d->Sigma();
//	d->normalize();

	if (data->hasCTFF()) {
		char st[BUF_SIZE],*t;
		strcpy(st,data->Name());
		string sc = string(data->Name());
		if(sc.size() > BUF_SIZE-24) {
			printf("The name %s is too long!!!", data->Name());
			exit(-1);
		}
		if ((t=strchr(st+3,' '))) *t=0;
		sprintf(st+strlen(st)," %d,%d",x0,y0);
		d->setCTFF(st);
	}
	else {
		char st[BUF_SIZE];
		sprintf(st,"%d,%d : %55s",x0,y0,data->Path());
		d->setName(st);
	}

	ptcl.remove(j);
	ptcl.insert(j,d);
	clipview->addData(d);
}

boxsize=x1; 
sprintf(t,"%d",x1); 
Size->setText(t);

fclose(in);
boxname=QString(file);
redisplayBoxes();
}

void Control::Msavebdb() {
int i;
ImShape *ss;
if (imview->nShapes()==0) { error(ERR_WARN,"Tried to save with empty box db"); return; }
QString defaultBoxfile = mgraph_id + QString(".box");
QString s( QFileDialog::getSaveFileName(defaultBoxfile,"*.box") ),t;
if ( s.isNull() ) return;
FILE *out=fopen(s.data(),"wb");
if (!out) return;

for (i=0; i<imview->nShapes(); i++) {
	ss=imview->shapeAt(i);
	fprintf(out,"%d\t%d\t%d\t%d\t%d\n",(int)ss->x0,(int)ss->y0,(int)ss->x1,(int)ss->y1,(int)ss->user2);
}

fclose(out);

}

void Control::Mgridbox() {
delAll();

clipview->setUpdatesEnabled(FALSE);
for (int x=boxsize*.25; x<data->xSize()-boxsize; x+=boxsize*6/5) {
	for (int y=boxsize*.25; y<data->ySize()-boxsize; y+=boxsize*6/5) {
		int j=newClip(x,y,boxsize,boxsize);
		EMData *d=ptcl.at(j);
		d->normalize();
		clipview->addData(d);
	}
}
clipview->setUpdatesEnabled(TRUE);

redisplayBoxes();
}

void Control::Mnewboxsize() {
QString r;
ImShape *ss;
int ns;
r=OneLiner::getOneLine("Enter the new box size","0");
ns=r.toInt();
if (ns<=3) return;

for (int i=0; i<imview->nShapes(); i++) {
	ss=imview->shapeAt(i);
	ss->x0+=(ss->x1-ns)/2;
	ss->y0+=(ss->y1-ns)/2;
	ss->x1=ns;
	ss->y1=ns;
	boxMoved(i);
}

redisplayBoxes();
}

void Control::closeEvent( QCloseEvent *e ) {
Mquit();
}	


void Control::Mquit() { 
// This saves the window geometries
	FILE *out = DFopenw("boxerwin");
	if (out) {
		fprintf(out,"%d %d %d %d\n",x(),y(),width(),height());
		fprintf(out,"%d %d %d %d\n",clipview->x(),clipview->y(),clipview->width(),clipview->height());
		fprintf(out,"%d %d %d %d\n",imview->geometry().x(),imview->geometry().y(),imview->geometry().width(),imview->geometry().height());
		fclose(out);	
	}


	exit(0); 
}

void Control::Mzerotonoise() {
float *d,s;
int i,n;

n=data->xSize()*data->ySize();
s=data->Sigma()*.9;
d=data->getData();
for (i=0; i<n; i++) {
	if (d[i]==0) d[i]=grand(0,s);
}
data->doneData();
}

void Control::clipClick(QMouseEvent *e,int i,int x,int y) {
ImShape *s;

imview->select.type=ImShape::NONE;
switch (mode) {
case SELECT:
	s=imview->shapeAt(i);
	if (!s) return;
	imview->setOrigin((s->x0+s->x1/2)*scale-imview->width()/2,
		(s->y0+s->y1/2)*scale-imview->height()/2);
	imview->hilight(i);
	break;
case REMOVE:
	delClip(i);
	redisplayBoxes();
	break;
}

}

void Control::delClip(int i) {
if (i>=ptcl.count()) return;
imview->delShape(i);
clipview->removeData(i);
ptcl.remove(i);
//imview->imageChanged();
imview->update();
}

void Control::delAll() {
imview->clearShapes();
clipview->clearData();
ptcl.clear();
//imview->imageChanged();
imview->update();
}

int Control::newClip(int x0,int y0,int x1, int y1, int n,int showclip,float rot) {
ImShape *s;
int i;
EMData *d;

	s=new ImShape;
	s->type=ImShape::DOTBOX;
	s->x0=x0;
	s->y0=y0;
	s->x1=x1;
	s->y1=y1;
	s->color=red;
	s->text="";
	s->user2=-3;
	i=imview->addShape(s,n);
	if (rot) {
		EMData *tmp=data->clip(x0-x1/4,y0-y1/4,x1*3/2,y1*3/2);
		tmp->setRAlign(rot);
		tmp->rotateAndTranslate();
		d=tmp->clip(x1/4,y1/4,x1,y1);
	}
	else d=data->clip(x0,y0,x1,y1);
	d->ufloat=d->Sigma();
	if (data->hasCTFF()) {
		char st[BUF_SIZE],*t;
		string sc = string(data->Name());
		if(sc.size() > BUF_SIZE-24) {
			printf("The name %s is too long!!!", data->Name());
			exit(-1);
		}
		strcpy(st,data->Name());
		if ((t=strchr(st+3,' '))) *t=0;
		sprintf(st+strlen(st)," %d,%d",x0,y0);
		d->setCTFF(st);
	}
	else {
		char st[BUF_SIZE];
		sprintf(st,"%d,%d : %55s",x0,y0,data->Path());
		d->setName(st);
	}
	if (n==-1) ptcl.append(d);	
	else ptcl.insert(n,d);
//	imview->imageChanged();
	imview->update();
	if (showclip) clipview->addData(ptcl.at(i),i);

return i;
}

void Control::imMoved(int x, int y) {
pan->setValue(x,y);
}

void Control::imResized(int w, int h) {
float xh,yh;

if (data->xSize()>data->ySize()) 
	pan->resize(140,140*data->ySize()/data->xSize());
else pan->resize(140*data->xSize()/data->ySize(),140);
pan->setXRange(0,data->xSize()*scale-w);
pan->setYRange(0,data->ySize()*scale-h);
if (data->xSize()*scale<w) xh=1.0;
else xh=w/(data->xSize()*scale);
if (data->ySize()*scale<h) yh=1.0;
else yh=h/(data->ySize()*scale);

pan->setHandleSize(xh,yh);

}

// This will remove the intermediate helices from a helix pair
// it returns the number of the first image of the pair
int Control::helixClean(int n) {
int i,r=n;
ImShape *s=imview->shapeAt(n);
if (!s) return 0;

while (n>=0 && s->user2!=-1) { n--; s=imview->shapeAt(n); }
if (!s) return 0;

// Delete old shapes for this endpoint
for (i=0; i<imview->nShapes(); i++) {
	ImShape *s = imview->shapeAt(i);
	if (!s) { error(ERR_WARN,"Nonexistent shape?"); continue; }
	if (s->user2==n) { delClip(i); i--; if (i<r) r--; }
}

return r;
}

void Control::updHelix(int mode) {
int i;

for (i=imview->nShapes()-1; i>=0; i--) {
	ImShape *s = imview->shapeAt(i);
	if (s->user2==-1) helixUpdate(i);
}

}

int Control::helixUpdate(int n) {
int i;

n=helixClean(n);

//printf("upd %d\n",n);
// Create new sequence of boxes for this endpoint
ImShape *s1,*s2;
s1=imview->shapeAt(n);
if (!s1) return 0;
if (s1->user2==-2) { s2=s1; s1=imview->shapeAt(n-1); n=n-1;}
else s2=imview->shapeAt(n+1);
if (!s1 || !s2) return 0;

if (olap>=boxsize) olap=boxsize-1;

int DX=s2->x0-s1->x0;
int DY=s2->y0-s1->y0;
int nb=ceil(hypot((float)DX,(float)DY)/(boxsize-olap));
float xsca=hypot((float)DX,(float)DY)/(boxsize-olap);
float ysca=DY/xsca;
xsca=DX/xsca;
int hmode=helixmode->currentItem();

int j= -1;
float ang=-(atan2(float(DY),float(DX))-PI/2.0);
int x0,y0;
switch (hmode) {
case 0:		// normal boxing
	for (i=1; i<nb; i++) {
		j=newClip(round(s1->x0+xsca*i),round(s1->y0+ysca*i),boxsize,boxsize,n+i,1);
		ImShape *s=imview->shapeAt(j);
		s->color=magenta;
		s->user2=n;
	}
	if (j>=0) {
		ImShape *s=imview->shapeAt(j);
		s2->x0=s->x0;
		s2->y0=s->y0;
		delClip(j);
		boxMoved(j);
	}
	break;
case 1:		// Rotate straight helical segments vertical
	x0=s1->x0;
	y0=s1->y0;
	delClip(n);
	for (i=0; i<=nb; i++) {
		j=newClip(round(x0+xsca*i),round(y0+ysca*i),boxsize,boxsize,n+i,1,ang);
		ImShape *s=imview->shapeAt(j);
		s->color=magenta;
		s->user2=n;
		if (i==0) { s->user2=-1; s->color=red; }
		if (i==nb) { s->user2=-2; s->color=red; }
	}
	if (j>=0) delClip(j+1);
	break;
case 2:		// follow curved helices, with ends anchored
	{
	int x1,y1;
	x0=s1->x0;
	y0=s1->y0;
	x1=s2->x0;
	y1=s2->y0;
	delClip(n);

	float bstlc=9.9e20,bsta,bstlc2=9e20,bsta2;

	//optimize the angle of the first box
	EMData *tmp2=data->clip(x0-boxsize/4,y0-boxsize/4,boxsize*3/2,boxsize*3/2);
	tmp2->normalize();
	tmp2->filter(0,boxsize/8.0,1);
	EMData *tmp=tmp2->copy();

	float a;
	float dr;
	for (a=ang-PI*20./180.; a<ang+PI*20./180.; a+=PI*1/180.) {
		tmp->setRAlign(a);
		tmp->rotateAndTranslate();
		EMData *tc=tmp->clip(boxsize/4,boxsize/4,boxsize,boxsize);
		tc->realFilter(101);
		float f=0;
		for (i=boxsize/2-4; i<=boxsize/2+4; i++) f+=tc->valueAt(i,0);
		if (f<bstlc) { bstlc=f; bsta=a; };
		delete tc;
	}
	delete tmp;
	delete tmp2;
	
	//optimize the angle of the last box
	tmp2=data->clip(x1-boxsize/4,y1-boxsize/4,boxsize*3/2,boxsize*3/2);
	tmp2->normalize();
	tmp2->filter(0,boxsize/8.0,1);
	tmp=tmp2->copy();

	for (a=ang-PI*20./180.; a<ang+PI*20./180.; a+=PI*1/180.) {
		tmp->setRAlign(a);
		tmp->rotateAndTranslate();
		EMData *tc=tmp->clip(boxsize/4,boxsize/4,boxsize,boxsize);
		tc->realFilter(101);
		float f=0;
		for (i=boxsize/2-4; i<=boxsize/2+4; i++) f+=tc->valueAt(i,0);
		if (f<bstlc2) { bstlc2=f; bsta2=a; };
		delete tc;
	}
	delete tmp;
	delete tmp2;

	// Generate a vertically rotated reference image
	tmp=data->clip(x0-boxsize/4,y0-boxsize/4,boxsize*3/2,boxsize*3/2);
	tmp->normalize();
	tmp->setRAlign(bsta);
	tmp->rotateAndTranslate();
	EMData *ref=tmp->clip(boxsize/4,boxsize/4,boxsize,boxsize);
	delete tmp;
	tmp=data->clip(x1-boxsize/4,y1-boxsize/4,boxsize*3/2,boxsize*3/2);
	tmp->normalize();
	tmp->setRAlign(bsta2);
	tmp->rotateAndTranslate();
	ref->add(tmp);
	EMData **refs = (EMData **)malloc(sizeof(EMData *)*41);
	delete tmp;
	ref->filter(0,boxsize/8,1);
	ref->writeIMAGIC("ref.hed",0);
	ref->realFilter(101);
	ref->writeIMAGIC("ref.hed",1);
	ref->realFilter(8,ref->Min()/1.5);
	ref->writeIMAGIC("ref.hed",2);
	ref->applyMask(-1,4);
	ref->writeIMAGIC("ref.hed",3);
	
//	printf("bsta=%f\n",bsta*180/PI);
	
	// Now generate a set of reference images over a +-10 degree range
	for (i=-20; i<=20; i++) {
		refs[i+20]=ref->copy();
		refs[i+20]->setRAlign(-bsta-i*.5*PI/180.);
		refs[i+20]->rotateAndTranslate();
//		refs[i+20]->normalize();
		refs[i+20]->ufloat2=bsta+i*.5*PI/180.;
	}
	refs[0]->writeIMAGIC("ref.hed",4);
	refs[40]->writeIMAGIC("ref.hed",5);
	
	float cx=x0-(boxsize-olap)*sin(bsta);
	float cy=y0-(boxsize-olap)*cos(bsta);
	float bstdr=bsta;

	// finally, trace the helix	
	for (i=0; i<=nb; i++) {
		ang=bstdr;
		bstlc=-1.0e32;
		for (a=ang-PI*5./180.; a<ang+PI*5./180.; a+=PI*.5/180.) {	// angle from last center to new center
			tmp2=data->clip(cx+(boxsize-olap)*sin(a),cy+(boxsize-olap)*cos(a),boxsize,boxsize);
			tmp2->filter(0,boxsize/8,1);
			tmp2->normalize();
			for (j=0; j<=40; j++) {		// angle of helix at current center
				float f=tmp2->dot(refs[j]);
				if (f>bstlc) { bstlc=f; bsta=a; bstdr=refs[j]->ufloat2; }
			}
			delete tmp2;
		}
		
//		printf("%1.1f , %1.1f\t%f\t%f\n",cx+(boxsize-olap)*sin(bsta),cy+(boxsize-olap)*cos(bsta),bsta*180./PI,bstdr*180./PI);
		j=newClip(cx+(boxsize-olap)*sin(bsta),cy+(boxsize-olap)*cos(bsta),boxsize,boxsize,n+i,1,bstdr);
		ImShape *s=imview->shapeAt(j);
		s->color=magenta;
		s->user2=n;
		if (i==0) { s->user2=-1; s->color=red; }
		if (i==nb) { s->user2=-2; s->color=red; }
		cx+=(boxsize-olap)*sin(bsta);
		cy+=(boxsize-olap)*cos(bsta);
	}
	if (j>=0) delClip(j+1);

	for (i=0; i<41; i++) delete refs[i];
	free(refs);
	delete ref;	
	}
	break;
case 3:		// full unbending with bilinear interpolation
	{
	x0=s1->x0;
	y0=s1->y0;
	delClip(n);
	EMData *tmp2=data->clip(x0-boxsize/4,y0-boxsize/4,boxsize*3/2,boxsize*3/2);
	tmp2->normalize();
	tmp2->filter(0,boxsize/8.0,1);
	EMData *tmp=tmp2->copy();

	//optimize the angle of the first box
	float bstlc=-9.9e20,bsta,bstdr;
	float a;
	float dr;
	for (a=ang-PI*10./180.; a<ang+PI*10./180.; a+=PI*1/180.) {
		tmp->setRAlign(a);
		tmp->rotateAndTranslate();
		EMData *tc=tmp->clip(boxsize/4,boxsize/4,boxsize,boxsize);
		tc->realFilter(101);
		float f=0;
		for (i=boxsize/4; i<=boxsize*3/4; i++) f+=SQR(tc->valueAt(i,0)-tc->valueAt(i-1,0));
		if (f>bstlc) { bstlc=f; bsta=a; };
		delete tc;
	}
	// This would optimize the helix center, but seemed to do more harm than good
/*	bstlc=9.9e20;
	for (dr=-5; dr<=5; dr++) {
		tmp->setRAlign(bsta);
		tmp->setTAlign(dr*sin(PI/2.0+a),dr*cos(PI/2.0+a));
		tmp->rotateAndTranslate();
		EMData *tc=tmp->clip(boxsize/4,boxsize/4,boxsize,boxsize);
		tc->realFilter(101);
		float f=0;
		for (i=boxsize/2-6; i<boxsize/2; i++) f+=SQR(tc->valueAt(i,0)-tc->valueAt(boxsize-i,0));
		if (f<bstlc) { bstlc=f; bstdr=dr; };
		delete tc;
	}*/
	bstdr=0;
	delete tmp;
	delete tmp2;
	
	// Generate a vertically rotated reference image
	tmp=data->clip(x0-boxsize/4-bstdr*sin(PI/2.0+bsta),y0-boxsize/4-bstdr*cos(PI/2.0+bsta),boxsize*3/2,boxsize*3/2);
	tmp->normalize();
	tmp->setRAlign(bsta);
	tmp->rotateAndTranslate();
	EMData *ref=tmp->clip(boxsize/4,boxsize/4,boxsize,boxsize);
	delete tmp;
	ref->filter(0,boxsize/8,1);
	ref->writeIMAGIC("ref.hed",0);
	ref->realFilter(101);
	ref->writeIMAGIC("ref.hed",1);
//	ref->realFilter(8,ref->Min()/1.5);
	ref->writeIMAGIC("ref.hed",2);
	ref->normalize();
	ref->applyMask(-1,4);
	ref->writeIMAGIC("ref.hed",3);
	
	printf("bsta=%f bstdr=%f\n",bsta*180/PI,bstdr);
	
	tmp=ref->copy();
		
	float cx=x0-bstdr*sin(PI/2.0+bsta)-(boxsize-olap)*sin(bsta);
	float cy=y0-bstdr*cos(PI/2.0+bsta)-(boxsize-olap)*cos(bsta);
	bstdr=bsta;

	float astp=atan2(1.0,boxsize/2);		// angular step in search = 1 pixel at edge of box
	// finally, trace the helix	
	for (i=0; i<=nb; i++) {
		ang=bstdr;
		bstlc=-1.0e32;
		for (a=ang-PI*5./180.; a<ang+PI*5./180.; a+=PI*.5/180.) {	// angle from last center to new center
			tmp2=data->clip(cx+(boxsize-olap)*sin(a),cy+(boxsize-olap)*cos(a),boxsize,boxsize);
			tmp2->filter(0,boxsize/8,1);
			tmp2->normalize();
			for (j=0; j<=12; j++) {		// angle of helix at current center
				tmp->setRAlign(-ang-(j-6)*astp);
				tmp->rotateAndTranslate();
				float f=tmp2->dot(tmp);
				if (f>bstlc) { bstlc=f; bsta=a; bstdr=ang+(j-6)*astp; }
			}
			delete tmp2;
		}
		
//		printf("%1.1f , %1.1f\t%f\t%f\n",cx+(boxsize-olap)*sin(bsta),cy+(boxsize-olap)*cos(bsta),bsta*180./PI,bstdr*180./PI);
		j=newClip(cx+(boxsize-olap)*sin(bsta),cy+(boxsize-olap)*cos(bsta),boxsize,boxsize,n+i,1,bstdr);
		ImShape *s=imview->shapeAt(j);
		s->color=magenta;
		s->user2=n;
		if (i==0) { s->user2=-1; s->color=red; }
		if (i==nb) { s->user2=-2; s->color=red; }
		cx+=(boxsize-olap)*sin(bsta);
		cy+=(boxsize-olap)*cos(bsta);

		// add the current segment into the reference for better alignment
		EMData *cp=ptcl.at(j)->copy(0,0);
		cp->filter(0,boxsize/8,1);
		cp->realFilter(101);
		cp->normalize();
		cp->applyMask(-1,4);
		ref->add(cp);
		delete cp;
	}
	if (j>=0) delClip(j+1);

	delete ref;
	delete tmp;
	}
	break;
case 4:		// simple version of following the path
	{
	x0=s1->x0;
	y0=s1->y0;
	delClip(n);
	EMData *tmp2=data->clip(x0-boxsize/4,y0-boxsize/4,boxsize*3/2,boxsize*3/2);
	tmp2->normalize();
	tmp2->filter(0,boxsize/8.0,1);
	EMData *tmp=tmp2->copy();

	//optimize the angle of the first box
	float bstlc=9.9e20,bsta,bstdr;
	float a;
	float dr;
	for (a=ang-PI*20./180.; a<ang+PI*20./180.; a+=PI*1/180.) {
		for (dr=-5; dr<=5; dr++) {
			tmp->setRAlign(a);
			tmp->setTAlign(dr*sin(PI/2.0+a),dr*cos(PI/2.0+a));
			tmp->rotateAndTranslate();
			EMData *tc=tmp->clip(boxsize/4,boxsize/4,boxsize,boxsize);
			tc->realFilter(101);
			float f=0;
			for (i=boxsize/2-4; i<=boxsize/2+4; i++) f+=tc->valueAt(i,0);
			if (f<bstlc) { bstlc=f; bsta=a; bstdr=dr; };
			delete tc;
		}
	}
	delete tmp;
	delete tmp2;
	
	// Generate a vertically rotated reference image
	tmp=data->clip(x0-boxsize/4-bstdr*sin(PI/2.0+bsta),y0-boxsize/4-bstdr*cos(PI/2.0+bsta),boxsize*3/2,boxsize*3/2);
	tmp->normalize();
	tmp->setRAlign(bsta);
	tmp->rotateAndTranslate();
	EMData *ref=tmp->clip(boxsize/4,boxsize/4,boxsize,boxsize);
	EMData **refs = (EMData **)malloc(sizeof(EMData *)*41);
	delete tmp;
	ref->filter(0,boxsize/8,1);
	ref->writeIMAGIC("ref.hed",0);
	ref->realFilter(101);
	ref->writeIMAGIC("ref.hed",1);
	ref->realFilter(8,ref->Min()/1.5);
	ref->writeIMAGIC("ref.hed",2);
	ref->applyMask(-1,4);
	ref->writeIMAGIC("ref.hed",3);
	
//	printf("bsta=%f\n",bsta*180/PI);
	
	// Now generate a set of reference images over a +-10 degree range
	for (i=-20; i<=20; i++) {
		refs[i+20]=ref->copy();
		refs[i+20]->setRAlign(-bsta-i*.5*PI/180.);
		refs[i+20]->rotateAndTranslate();
//		refs[i+20]->normalize();
		refs[i+20]->ufloat2=bsta+i*.5*PI/180.;
	}
	refs[0]->writeIMAGIC("ref.hed",4);
	refs[40]->writeIMAGIC("ref.hed",5);
	
	float cx=x0-bstdr*sin(PI/2.0+bsta)-(boxsize-olap)*sin(bsta);
	float cy=y0-bstdr*cos(PI/2.0+bsta)-(boxsize-olap)*cos(bsta);
	bstdr=bsta;

	// finally, trace the helix	
	for (i=0; i<=nb; i++) {
		ang=bstdr;
		bstlc=-1.0e32;
		for (a=ang-PI*5./180.; a<ang+PI*5./180.; a+=PI*.5/180.) {	// angle from last center to new center
			tmp2=data->clip(cx+(boxsize-olap)*sin(a),cy+(boxsize-olap)*cos(a),boxsize,boxsize);
			tmp2->filter(0,boxsize/8,1);
			tmp2->normalize();
			for (j=0; j<=40; j++) {		// angle of helix at current center
				float f=tmp2->dot(refs[j]);
				if (f>bstlc) { bstlc=f; bsta=a; bstdr=refs[j]->ufloat2; }
			}
			delete tmp2;
		}
		
//		printf("%1.1f , %1.1f\t%f\t%f\n",cx+(boxsize-olap)*sin(bsta),cy+(boxsize-olap)*cos(bsta),bsta*180./PI,bstdr*180./PI);
		j=newClip(cx+(boxsize-olap)*sin(bsta),cy+(boxsize-olap)*cos(bsta),boxsize,boxsize,n+i,1,bstdr);
		ImShape *s=imview->shapeAt(j);
		s->color=magenta;
		s->user2=n;
		if (i==0) { s->user2=-1; s->color=red; }
		if (i==nb) { s->user2=-2; s->color=red; }
		cx+=(boxsize-olap)*sin(bsta);
		cy+=(boxsize-olap)*cos(bsta);
	}
	if (j>=0) delClip(j+1);

	for (i=0; i<41; i++) delete refs[i];
	free(refs);
	delete ref;	
	}
	break;
}

/*
printf("---\n");
for (i=0; i<imview->nShapes(); i++) {
	ImShape *s=imview->shapeAt(i);
	printf("%d. %f %f %d\n",i,s->x0,s->y0,s->user2);
}
*/
return j;
}

void Control::imDown(QMouseEvent *e) {

//when no image loaded, mouse click will crash boxer, this check fix this problem
if(data->xSize() == 2 || data->ySize() == 2 || data->zSize() == 2) {
	return;
}

int n=imview->nShapes();
int i,j;
ImShape *s;

if (curshape!=-1) printf("Down before up!\n");

mx0=e->x();
my0=e->y();

imview->select.type=ImShape::NONE;

switch (mode) {
case MEASURE:
	s=new ImShape;
	s->type=ImShape::LINE;
	s->x1=s->x0=imview->pxtodx(e->x());
	s->y1=s->y0=imview->pytody(e->y());
	s->text="";
	s->color=red;
	s->user2=-3;
	curshape=imview->addShape(s);
	break;
case SELECT:
	// see if click was inside existing box
	for (i=0; i<n ; i++) {
		s=imview->shapeAt(i);
		if (e->x()>imview->dxtopx(s->x0) && e->x()<imview->dxtopx(s->x0+s->x1) &&
			e->y()<imview->dytopy(s->y0) && e->y()>imview->dytopy(s->y0+s->y1)) {
			curshape=i;
			sx0=s->x0;
			sy0=s->y0;
			break;
		}
	}
	if (i!=n) {
		imview->hilight(curshape);
		break;
	}
	// if not, make a new one
	curshape=newClip(imview->pxtodx(e->x())-boxsize/2,imview->pytody(e->y())-boxsize/2,
		boxsize,boxsize);
	sx0=imview->pxtodx(e->x())-boxsize/2;
	sy0=imview->pytody(e->y())-boxsize/2;
	s=imview->shapeAt(curshape);
	s->user2=-3;
	imview->hilight(curshape);
//printf("select %d %d   %d %d\n",mx0,my0,sx0,sy0);
	break;
case HELIX:
	// see if click was inside existing box

	j=0;
	for (i=0; i<n ; i++) {
		s=imview->shapeAt(i);
		if (e->x()>imview->dxtopx(s->x0) && e->x()<imview->dxtopx(s->x0+s->x1) &&
			e->y()<imview->dytopy(s->y0) && e->y()>imview->dytopy(s->y0+s->y1)) {
			if ((s->user2==-1||s->user2==-2)) {
				curshape=i;
				sx0=s->x0;
				sy0=s->y0;
				break;
			}
			j=1;
		}
	}
	if (i!=n) {
//		helixClean(curshape);
		imview->hilight(curshape);
//DBUG("s %d. %f,%f  %d\n",i,s->x0,s->y0,s->user2);
		break;
	}
	if (j) break;
	
	// if not, make a new one
	curshape=newClip(imview->pxtodx(e->x())-boxsize/2,imview->pytody(e->y())-boxsize/2,
		boxsize,boxsize);

	s=imview->shapeAt(curshape-1);
	if (s && s->user2==-1) {
		s=imview->shapeAt(curshape);
		if (s) s->user2=-2;
	}
	else {
		s=imview->shapeAt(curshape);
		if (s) s->user2=-1;
	}
	sx0=imview->pxtodx(e->x())-boxsize/2;
	sy0=imview->pytody(e->y())-boxsize/2;
	curshape=helixClean(curshape);

	imview->hilight(curshape);
	break;
case REMOVE:
	for (i=0; i<n ; i++) {
		s=imview->shapeAt(i);
		if (e->x()>imview->dxtopx(s->x0) && e->x()<imview->dxtopx(s->x0+s->x1) &&
			e->y()<imview->dytopy(s->y0) && e->y()>imview->dytopy(s->y0+s->y1)) break;
	}
	if (i==n) break;
	if (s->user2==-1 || s->user2==-2) {
		while (s->user2!=-1) { i--; s=imview->shapeAt(i); }
		i=helixClean(i); 
		delClip(i); 
		delClip(i); 
	}
	else delClip(i);
	redisplayBoxes();
	break;
case DRAW:
	int j,k,nx,ny,xo,yo,x0,x1,y0,y1;
	float *d;
	nx=data->xSize();
	ny=data->ySize();
	
	xo=imview->pxtodx(e->x());
	yo=imview->pytody(e->y());

	x0=xo-boxsize/2;
	x1=xo+boxsize/2;
	y0=yo-boxsize/2;
	y1=yo+boxsize/2;
	
	if (x0<0) x0=0;
	if (y0<0) y0=0;
	if (x1>nx-1) x1=nx-1;
	if (y1>ny-1) y1=ny-1;

	d=data->getDataRO();
	for (j=y0; j<y1; j++) {
		for (i=x0; i<x1; i++) {
			if ((e->state()&ControlButton) || hypot((float)i-xo,(float)j-yo)<boxsize/2 ) d[i+j*nx]=0;
		}
	}
	data->doneData();
	imview->imageChanged();
	break;
}

}

void Control::imMove(QMouseEvent *e) {
ImShape *s;
int n=imview->nShapes();
int i,j;

if (curshape<0 && mode!=DRAW && mode!=REMOVE) return;

switch (mode) {
case MEASURE:
	s=imview->shapeAt(curshape);
	s->x1=imview->pxtodx(e->x());
	s->y1=imview->pytody(e->y());
	s->text.sprintf("%d pix %1.2f A",(int)hypot(s->x1-s->x0,s->y1-s->y0),hypot(s->x1-s->x0,s->y1-s->y0)*apix);
//	imview->imageChanged();
	imview->update();
	break;
case SELECT:
case HELIX:
	s=imview->shapeAt(curshape);
	s->x0=(int)floor((e->x()-mx0)/imview->lscale+sx0);
	s->y0=(int)floor(-(e->y()-my0)/imview->lscale+sy0);
//	imview->imageChanged();
	imview->hilight(curshape);
	imview->update();
	break;
case REMOVE:
	for (i=0; i<n ; i++) {
		s=imview->shapeAt(i);
		if (e->x()>imview->dxtopx(s->x0) && e->x()<imview->dxtopx(s->x0+s->x1) &&
			e->y()<imview->dytopy(s->y0) && e->y()>imview->dytopy(s->y0+s->y1)) break;
	}
	if (i==n) break;
	if (s->user2==-1 || s->user2==-2) {
		while (s->user2!=-1) { i--; s=imview->shapeAt(i); }
		i=helixClean(i); 
		delClip(i); 
		delClip(i); 
	}
	else delClip(i);
	redisplayBoxes();

	return;
case DRAW:
	int i,j,k,nx,ny,xo,yo,x0,x1,y0,y1;
	float *d;
	nx=data->xSize();
	ny=data->ySize();
	
	xo=imview->pxtodx(e->x());
	yo=imview->pytody(e->y());

	x0=xo-boxsize/2;
	x1=xo+boxsize/2;
	y0=yo-boxsize/2;
	y1=yo+boxsize/2;
	
	if (x0<0) x0=0;
	if (y0<0) y0=0;
	if (x1>nx-1) x1=nx-1;
	if (y1>ny-1) y1=ny-1;

	d=data->getDataRO();
	for (j=y0; j<y1; j++) {
		for (i=x0; i<x1; i++) {
			if ((e->state()&ControlButton) || hypot((float)i-xo,(float)j-yo)<boxsize/2 ) d[i+j*nx]=0;
		}
	}
	data->doneData();
	imview->imageChanged();
	break;
}

}

void Control::imUp(QMouseEvent *e) {
ImShape *s;
EMData *d;

if (curshape<0) return;

switch (mode) {
case MEASURE:
	imview->delShape(curshape);
	curshape=-1;
	break;
case SELECT:
	s=imview->shapeAt(curshape);
	s->x0=(int)floor((e->x()-mx0)/imview->lscale+sx0);
	s->y0=(int)floor(-(e->y()-my0)/imview->lscale+sy0);
//	imview->imageChanged();
	imview->update();
	boxMoved(curshape);
	redisplayBoxes(curshape);
	curshape=-1;
	break;
case HELIX:
	s=imview->shapeAt(curshape);
	if (!s) { curshape=-1; break; }
	s->x0=(int)floor((e->x()-mx0)/imview->lscale+sx0);
	s->y0=(int)floor(-(e->y()-my0)/imview->lscale+sy0);

	boxMoved(curshape);
	curshape=helixUpdate(curshape);
	imview->hilight(curshape);
	imview->update();
	redisplayBoxes(curshape);
	curshape=-1;

	break;	
case REMOVE:
	return;
}


}

void Control::boxMoved(int n) {
	ImShape *s;
	EMData *d;

	s=imview->shapeAt(n);
	d=data->clip(s->x0,s->y0,s->x1,s->y1);
	d->ufloat=d->Sigma();
//	d->normalize();
	if (data->hasCTFF()) {
		char st[BUF_SIZE],*t;
		string sc = string(data->Name());
		if(sc.size() > BUF_SIZE-24) {
			printf("The name %s is too long!!!", data->Name());
			exit(-1);
		}
		strcpy(st,data->Name());
		if ((t=strchr(st+3,' '))) *t=0;
		sprintf(st+strlen(st)," %1.0f,%1.0f",s->x0,s->y0);
		d->setCTFF(st);
	}
	else {
		char st[BUF_SIZE];
		sprintf(st,"%1.0f,%1.0f : %55s",s->x0,s->y0,data->Path());
		d->setName(st);
	}
	clipview->replaceData(n,d);
	clipview->makeVisible(n);
	ptcl.remove(n);
	ptcl.insert(n,d);
}

void Control::redisplayBoxes(int n) {
clipview->viewport()->update();

}

#define AUTOCLIP 1024
void Control::Mfocalauto() {
int r=QMessageBox::information(NULL,"Important note","This option will take box positions from one image in\
 a focal pair\nand adjust the box positions to fit the second image. The machine MUST have enough\n\
 memory to hold BOTH micrographs at once. To use this option, you should have already boxed\n\
and saved the particles from the first image. When you close this window, a\n\
file dialog will appear. Select the OTHER image (the one you haven't boxed). Boxer will\n\
adjust the box positions, and the new image will be displayed on the screen.");
 
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;

int nx=data->xSize(),ny=data->ySize();
QProgressDialog progress( "Realigning boxes", 0, ((nx-1)/AUTOCLIP+1)*((ny-1)/AUTOCLIP+1)+1, this, 0 ,TRUE );
progress.setProgress( 0 );

EMData *pair=new EMData;
EMData *clip, *clipp, *clipc, *t;
if (pair->readImage(s,0)) return;
pair->normalize();
int nx2=pair->xSize(),ny2=pair->ySize();
printf("Read image '%s'\n",(const char *)s);

int i,j,k,pr=1;
for (i=0; i<imview->nShapes(); i++) imview->shapeAt(i)->user=0;

float da,bda,bdx,bdy,pk;
for (i=0; i<nx; i+=AUTOCLIP) {
	for (j=0; j<ny; j+=AUTOCLIP) {
		progress.setProgress(pr++);
		Olist best(3);
		if (i+AUTOCLIP>nx) i=nx-AUTOCLIP;
		if (j+AUTOCLIP>ny) j=ny-AUTOCLIP;
		t=data->clip(i,j,AUTOCLIP,AUTOCLIP);
		t->medianShrink(2);
		t->filter(3.0,AUTOCLIP/8,1);
		clip=t->clip(AUTOCLIP/16,AUTOCLIP/16,AUTOCLIP*3/8 ,AUTOCLIP*3/8);
		clip->applyMask(-1,4);
		clipc=clip->copy();
		delete t;

		t=pair->clip(i,j,AUTOCLIP,AUTOCLIP);
		t->medianShrink(2);
		t->filter(3.0,AUTOCLIP/8,1);
		clipp=t->clip(AUTOCLIP/16,AUTOCLIP/16,AUTOCLIP*3/8 ,AUTOCLIP*3/8);
		clipp->applyMask(-1,4);
		clipp->normalize();

		for (da=-10.0*PI/180.0; da<10.0*PI/180.0; da+=2.0*PI/180.0) {
			clipc->setRAlign(da);
			clipc->setTAlign(0,0,0);
			clipc->rotateAndTranslate();
			clipc->normalize();
			clipc->transAlign(clipp,0,1,nx/4,&pk);
			best.insert(pk,clipc->Dx(),clipc->Dy(),da);
		}
		bda=best.da[0];
		for (da=bda-2.0*PI/180.0; da<bda+2.0*PI/180.0; da+=.5*PI/180.0) {
			clipc->setRAlign(da);
			clipc->setTAlign(0,0,0);
			clipc->rotateAndTranslate();
			clipc->normalize();
			clipc->transAlign(clipp,0,1,nx/4,&pk);
			best.insert(pk,clipc->Dx()*cos(da)+clipc->Dy()*sin(da),
				-clipc->Dx()*sin(da)+clipc->Dy()*cos(da),da);
		}
		bda=best.da[0];
		bdx=best.dx[0]*2;
		bdy=best.dy[0]*2;
		
		printf("(%d,%d)\t%1.1f (%1.0f,%1.0f)\n",i,j,bda*180.0/PI,best.dx[0],best.dy[0]);

		float mx0=cos(bda);
		float mx1=sin(bda);
		ImShape *sh;
		float x,y;
		for (k=0; k<imview->nShapes(); k++) {
			sh=imview->shapeAt(k);
			if (sh->user) continue;
			x=sh->x0-i-AUTOCLIP/2+sh->x1/2+bdx;
			y=sh->y0-j-AUTOCLIP/2+sh->y1/2+bdy;
			if (x<-AUTOCLIP/2||x>AUTOCLIP/2 || y<-AUTOCLIP/2||y>AUTOCLIP/2) continue;
			sh->x0=(x*mx0-y*mx1)+i+AUTOCLIP/2-sh->x1/2;
			sh->y0=(x*mx1+y*mx0)+j+AUTOCLIP/2-sh->y1/2;
			sh->user=1;
//printf("%1.0f,%1.0f -> %d,%d\n",x+i+AUTOCLIP/2-sh->x1/2,y+j+AUTOCLIP/2-sh->y1/2,sh->x0,sh->y0);
			if (sh->x0<0 || sh->y0<0 || sh->x0+sh->x1>nx || sh->y0+sh->y1>ny) {
				delClip(k);
				k--;
			}
		}
		
		// for testing only
/*		clipc->setRAlign(bda);
		clipc->setTAlign(bdx/2,bdy/2,0);
		clipc->rotateAndTranslate();
		clipc->normalize();
		clipc->writeIMAGIC("tst.hed",-1);
		clipp->writeIMAGIC("tst.hed",-1);
		EMData *e=clipc->calcCCF(clipp);
		e->writeIMAGIC("tst.hed",-1);*/

		delete clipc;
		delete clip;
		delete clipp;
	}
}

delete data;
data=pair;

imview->setData(data);
imview->update();
//imview->imageChanged();
imResized(imview->width(),imview->height());

for (k=0; k<imview->nShapes(); k++) boxMoved(k);

redisplayBoxes();
}

struct sRect { int x,y,x2,y2; };

// This was cut from another program, so it's sort of bizzare
void Control::adjustBoxes() {
int i,j,n=imview->nShapes();
sRect *rect;
float dx=0,dy=0,nm=0,ma=0,f,xAve=0, yAve=0, xNewAve=0.,yNewAve=0.;
int x0,y0;

rect=(sRect *)malloc(sizeof(sRect)*imview->nShapes());

// First read original (untranslated) box database file
//QString s( QFileDialog::getOpenFileName("Original box database file") );
QString s(boxname);
if ( s.isNull() ) return;
FILE *in=fopen(s.data(),"rb");
if (!in) return;

char t[120];
for (i=0; fgets(t,119,in) && sscanf(t," %d %d %d %d",&x0,&y0,&j,&j)==4; i++) {
	rect[i].x=x0;
	rect[i].y=y0;

	if (i>=n) { printf("You MUST use the original box database file (%d != %d)\n",i,n); exit(1); }
}
if (i<n) { printf(" %d != %d\n",i,n); return; }

fclose(in);

// Then set the adjusted box positions
for (i=0; i<n; i++) {
	rect[i].x2=imview->shapeAt(i)->x0;
	rect[i].y2=imview->shapeAt(i)->y0;
	if (rect[i].x==rect[i].x2 && rect[i].y==rect[i].y2) { rect[i].x2=rect[i].y2=-1; }
}

// now do a translational shift
nm=0;
for (i=0; i<n; i++) {
	if (rect[i].x2>=0 || rect[i].y2>=0) {
		xAve +=(float)rect[i].x;     // average positions before
		yAve +=(float)rect[i].y;
		xNewAve += (float)rect[i].x2; // ave positions after
		yNewAve += (float)rect[i].y2;
		nm+=1.0;
	}
}
xAve =   xAve/nm;
yAve =   yAve/nm;
xNewAve = xNewAve/nm;
yNewAve = yNewAve/nm;

float OOXX=0,  OOXY=0,  OOYX=0,  OOYY=0;  // old old matrix
float NOXX=0,  NOXY=0,  NOYX=0,  NOYY=0;  // new new matrix
float M11, M12, M21, M22, deltaX, deltaY;
float x1, y1, x2, y2;


if (nm==0) { printf("None of the boxes moved. Nothing to do.\n"); return; }

if (nm==1) { 
	//printf("Only doing translations");
	M11 =1; M12 =0; M21 =0; M22 = 1;
}

if (nm==2) { 
	//printf("Only doing translations, rotation, rescale");
	for (i=0; i<n; i++) {
		if (rect[i].x2>=0 || rect[i].y2>=0) {
			x1  = (rect[i].x  - xAve);
			x2  = (rect[i].x2 - xNewAve);
			y1  = (rect[i].y  - yAve);
			y2  = (rect[i].y2 - yNewAve);
			continue;
		}
	}
	M11 = (x1*x2+y1*y2)/(x1*x1+y1*y1) ; 
	M12 = (y1*x2-y2*x1)/(x1*x1+y1*y1) ; 
	M21 = -M12; 
	M22 =  M11;
}

// Change in coordinate is NewAve - Ave + (M-1)(Old_i -Ave)

if (nm>=3) { 
	//printf("Doing Full Linear Transformation and Translation");
	for (i=0; i<n; i++) {
		if (rect[i].x2>=0 || rect[i].y2>=0) {
			OOXX +=  (rect[i].x  -xAve   )*(rect[i].x -xAve); 
			OOXY +=  (rect[i].x  -xAve   )*(rect[i].y -yAve); 
			OOYX +=  (rect[i].y  -yAve   )*(rect[i].x -xAve); 
			OOYY +=  (rect[i].y  -yAve   )*(rect[i].y -yAve); 
			NOXX +=  (rect[i].x2 -xNewAve)*(rect[i].x -xAve); 
			NOXY +=  (rect[i].x2 -xNewAve)*(rect[i].y -yAve); 
			NOYX +=  (rect[i].y2 -yNewAve)*(rect[i].x -xAve); 
			NOYY +=  (rect[i].y2 -yNewAve)*(rect[i].y -yAve); 
			continue;
		}
		M11 = ( NOXX*OOYY-NOXY* OOYX) /(OOXX*OOYY - OOXY*OOYX) ; 
		M12 = (-NOXX*OOXY+NOXY* OOXX) /(OOXX*OOYY - OOXY*OOYX)  ; 
		M21 = ( NOYX*OOYY-NOYY* OOYX) /(OOXX*OOYY  - OOXY*OOYX); 
		M22 = (-NOYX*OOXY+NOYY* OOXX) /(OOXX*OOYY  - OOXY*OOYX); 
	}
}

	for (i=0; i<n; i++) {
		deltaX = rect[i].x - xAve;
		deltaY = rect[i].y - yAve;
		rect[i].x += xNewAve  -xAve + (M11-1)*deltaX +  M12*deltaY;
		rect[i].y += yNewAve  -yAve +  M21 *deltaX   + (M22-1)*deltaY;
		rect[i].x2 = -1;
		rect[i].y2 = -1;
	}

// now write the shifted boxes back to the original array
for (i=j=0; i<n; i++,j++) {
	imview->shapeAt(i)->x0=rect[j].x;
	imview->shapeAt(i)->y0=rect[j].y;
//printf("%d %d   %d %d\n",rect[j].x,rect[j].y,rect[j].x+imview->shapeAt(i)->x1,rect[j].y+imview->shapeAt(i)->y1);
	if (rect[j].x<0 || rect[j].y<0 || rect[j].x+imview->shapeAt(i)->x1>data->xSize() 
			|| rect[j].y+imview->shapeAt(i)->y1>data->ySize() ) {
		delClip(i);
		i--;
		n--;
	}
	else boxMoved(i);
}
//imview->imageChanged();
imview->update();

redisplayBoxes();
free(rect);
}


void Control::autoMask(int first,int last,float thr,float filt) {
int i;

if (last>ptcl.count()-1) last=ptcl.count()-1;
if (first<0) first=0;
if (ptcl.count()==0) return;

QApplication::setOverrideCursor( waitCursor );
QProgressDialog progress( "Automasking", "Abort", last-first, this, 0 ,TRUE );
progress.setProgress( 0 );

for (i=first; i<=last; i++) {
	boxMoved(i);
	EMData *d=ptcl.at(i);
	d->autoMask(thr,filt);
	progress.setProgress( i-first );
	if ( progress.wasCancelled() ) break;
}
progress.setProgress(last-first);

QApplication::restoreOverrideCursor( );

}

void Control::Munautomask() {

for (int i=0; i<ptcl.count(); i++) boxMoved(i);
}

void Control::Mautomask() {
if (amd) { amd->show(); return; }
amd=new AutomaskD;
amd->setTarget(this);
amd->show();
amd->newVal(0);
}

// this defines how large an area to autobox at once (*2)
//#define CLIP 512
int CLIP=768;

void Control::autoBox1() {
EMData *d1,*d2;
AutoboxD *ab;
int px,py;
ImShape *s;

if (ptcl.count()<1) {
	QMessageBox::information(this,"Usage","Before selecting Autobox, manually pick\na few representative particles representing various views.",
		"Ok");
	return;
}

QApplication::setOverrideCursor( waitCursor );

data->normalize();

abptcl.clear();
abvec.clear();
absig=0;
for (d1=ptcl.first(); d1; d1=ptcl.next()) {

	absig+=d1->ufloat;

	abvec.append(new RVEC(d1));
	d1->edgeNormalize();
	d1->radialAverage();
	d1->applyMask(-1,4);
	d2=d1->copy();
	d2->setParent(NULL);
	d2->medianShrink(2);
	d2->unum3=0;
	abptcl.append(d2);
}
absig/=abptcl.count();

abmm=data->Max()-data->Min();

s=imview->shapeAt(0);
px=s->x0+s->x1/2-512;
py=s->y0+s->y1/2-512;
if (px<0) px=0;
if (py<0) py=0;
if (px>data->xSize()-1024) px=data->xSize()-1025;
if (py>data->ySize()-1024) py=data->ySize()-1025;
d1=ptclCorr(px,py);

//d1->writeMRC("tstb.mrc");
ab=new AutoboxD();
ab->setup(px,py,d1->Min(),d1->Max(),d1,this);
ab->show();

QApplication::restoreOverrideCursor( );
}

void Control::autoBoxS() {
EMData *d1;		// min sigma array
EMData *d2;		// single sigma array
EMData *d3;		// micrograph
EMData *d4;		// iterated reference
int px,py;
ImShape *s;

if (ptcl.count()<1) {
	QMessageBox::information(this,"Usage","Before selecting Autobox, manually pick\na few representative particles representing various views.",
		"Ok");
	return;
}

data->normalize();

QApplication::setOverrideCursor( waitCursor );

int shfac=ptcl.first()->xSize()/32;
printf("Shrink factor = %d\n",shfac);

// Shrink the prototypes
abptcl.clear();
abvec.clear();
absig=0;
for (d1=ptcl.first(); d1; d1=ptcl.next()) {
	abvec.append(new RVEC(d1));
	d1->edgeNormalize();
	d1->radialAverage();
	d1->filter(-10,d1->xSize()/6,1);
	d1->applyMask(-1,4);
	d2=d1->copy(0,0);
	d2->meanShrink(shfac);
	abptcl.append(d2);
	d2->unum3=0;
}
delAll();

// calculate the 'best' correlation map (d1)
d3=data;
d3->meanShrink(shfac);
//d3->realFilter(17,.8,.8);
//d3->realFilter(10);

QString st(data->Path());
d3->writeMRC("shrink.mrc");
d3->setPath((const char *)st);

d1=d3->copyHead();
d1->zero();
//d1->addConst(1.e10);
for (d4=abptcl.first(); d4; d4=abptcl.next()) {
	d2=d3->littleBigDot(d4,0);
	
	for (int j=0; j<d3->ySize(); j++) {
		for (int i=0; i<d3->xSize(); i++) {
			if (d2->valueAt(i,j)>d1->valueAt(i,j)) d1->setValueAt(i,j,0,d2->valueAt(i,j));
		}
	}
	delete d2;
}

d1->update();
//d1->realFilter(90);
d1->writeMRC("dot.mrc");
printf("Reread image '%s'\n",data->Path());
data->readImage(data->Path(),0);
delete d1;
QApplication::restoreOverrideCursor( );
}

// This uses a set of reference projections for automatic boxing
void Control::autoBoxM() {
EMData *d1,*d2;
AutoboxD *ab;
int px,py,i;
ImShape *s;
List *refs;


QString f( QFileDialog::getOpenFileName() );
if ( f.isNull() ) return;
refs=readImages(f.data(),-1,-1);

QApplication::setOverrideCursor( waitCursor );
if (refs->count()<1) return;
if (refs->count()>250) error(ERR_EXIT,"FATAL ERROR: Max # references 250.");
Size->setText(QString::number(((EMData *)refs->objectAt(0))->xSize()));

Mnormalize();

abptcl.clear();
abvec.clear();
absig=0;
for (i=0; i<refs->count(); i++) {
	d1=(EMData *)refs->objectAt(i);
	d1->edgeNormalize();
//	d1->applyMask(-1,4);
	
	d2=d1->copy(0,0);
	d2->medianShrink(2);
	d2->unum3=0;
	abptcl.append(d2);
	delete d1;
}
delete refs;


abmm=data->Max()-data->Min();

px=imview->getOriginx();
py=imview->getOriginy();
d1=ptclCorr(px,py);

//d1->writeMRC("tstb.mrc");
ab=new AutoboxD();
printf("new complete\n");
ab->setup(px,py,d1->Min(),d1->Max(),d1,this);
printf("Setup complete\n");
ab->show();
QApplication::restoreOverrideCursor( );
}

// This uses a set of reference projections for automatic boxing
void Control::autoBoxSVD() {
EMData *d1,*d2;
AutoboxD *ab;
int px,py,i;
ImShape *s;
List *refs;


QString f( QFileDialog::getOpenFileName() );
if ( f.isNull() ) return;
refs=readImages(f.data(),-1,-1);

QApplication::setOverrideCursor( waitCursor );
if (refs->count()<1) return;
if (refs->count()>250) error(ERR_EXIT,"FATAL ERROR: Max # references 250.");
Size->setText(QString::number(((EMData *)refs->objectAt(0))->xSize()));

abptcl.clear();
abvec.clear();
absig=0;
for (i=0; i<refs->count(); i++) {
	d1=(EMData *)refs->objectAt(i);
	d2=d1->copy(0,0);
	d2->medianShrink(2);
	d2->setNImg(d1->NImg());
	if (d2->NImg()<=0) d2->setNImg(1.0);
	d2->unum3=1;
	abptcl.append(d2);
	delete d1;
}
delete refs;

abmm=data->Max()-data->Min();

px=imview->getOriginx();
py=imview->getOriginy();
d1=ptclCorrSVD(px,py);

//d1->writeMRC("tstb.mrc");
ab=new AutoboxD();
ab->setup(px,py,d1->Min(),d1->Max(),d1,this);
ab->show();
QApplication::restoreOverrideCursor( );
}


void Control::autoBox2(float *thresh) {
EMData *d1,*d2,*da;
int i,j,nx,ny;

nx=(data->xSize()-boxsize*2)/(CLIP*2-boxsize)+1;
ny=(data->ySize()-boxsize*2)/(CLIP*2-boxsize)+1;

delAll();

int SVD=((EMData *)abptcl.at(0))->unum3;

QProgressDialog progress( "Autoboxing", "Abort", nx*ny, this, 0 ,TRUE );
progress.setProgress( 0 );

for (i=0; i<ny; i++) {
	for (j=0; j<nx; j++) {
		progress.setProgress( i*nx+j );
		if (SVD) d1=ptclCorrSVD(j*(CLIP*2-boxsize),i*(CLIP*2-boxsize));
		else d1=ptclCorr(j*(CLIP*2-boxsize),i*(CLIP*2-boxsize));
		boxesFrom(d1,j*(CLIP*2-boxsize),i*(CLIP*2-boxsize),0,0,thresh);
		delete d1;
		if ( progress.wasCancelled() ) return;
	}
}

}

void Control::boxesFrom(EMData *map,int x0,int y0,int sx,int sy,float *thresh) {
int i,j,k,l,c,n=0,mn;
float *d,*d2,da1,da2,bd;
EMData *dt,*dtc;

if (!map) return;
clipview->setUpdatesEnabled(FALSE);

//if (x0==0 && y0==0) delAll();

// sx and sy allows overlaping areas to be dealt with
if (sx<boxsize/4) sx=boxsize/4;
if (sy<boxsize/4) sy=boxsize/4;

mn=4*CLIP*CLIP*4/(boxsize*boxsize);

d=map->getData();
for (j=sy; j<CLIP-boxsize/4; j++) {
	for (i=sx; i<CLIP-boxsize/4; i++) {
		if (d[i+j*CLIP]>thresh[0]) {
			if (n>mn ||i*2+x0>data->xSize()-boxsize || j*2+y0>data->ySize()-boxsize) continue;
			dt=data->clip(i*2+x0-boxsize/2,j*2+y0-boxsize/2,boxsize,boxsize);

			// here we check for particles with large 'drawn' areas and eliminate them
			d2=dt->getDataRO();	
			c=0;
			for (l=0; l<boxsize*boxsize; l++) {
				if (d2[l]==0) c++;
			}
			dt->doneData();
			if (c>boxsize*2) {
				delete dt;
				continue;
			}			

			// Make sure the particle really looks like the reference
			if (thresh[1]<1.0 || thresh[2]>0) {
				EMData *cp=dt->copy(0,0),*rf=((EMData *)abptcl.at(bestmap[i+j*CLIP]))->copy(0,0);
				cp->medianShrink(2);
				float mult,add;
				rf->normalizeTo(cp,0,0,&mult,&add);
//cp->writeIMAGIC("test.hed",-1);
//rf->writeIMAGIC("test.hed",-1);
				cp->subtract(rf);
				
				// now calculate the Kurtosis of the resulting image (0 for gaussians)
				float mean=cp->Mean(),sig=cp->Sigma(),kurt=0;
				float *d=cp->getDataRO();
				for (l=0; l<boxsize*boxsize/4; l++) kurt+=pow((d[l]-mean)/sig,4.0F);
				kurt=kurt/(boxsize*boxsize/4.0)-3.0;
				cp->doneData();
//printf("Kurt= %f\n",kurt);

				if (fabs(kurt)>thresh[3] || mult<thresh[2] || cp->Sigma()>thresh[1] || mult==1) { delete cp; delete rf; delete dt; continue; }
				
				delete cp;
				delete rf;
			}
//printf("%d,%d. %f\t%f\t%f\n",i,j,dt->Sigma(),d[i+j*CLIP]/dt->Sigma(),thresh[1]);		
//			if (d[i+j*CLIP]/SQR(dt->Sigma())<thresh[1]) {
//				delete dt;
//				continue;
//			}

/*
			// use the second & third thresholds as a sigma variation tolerance
			dtc=dt->clip(boxsize/4,boxsize/4,boxsize/2,boxsize/2);
//			da1=SQR((1.0-thresh[1])/2.0)*absig;
			da2=SQR((1.0-thresh[2])/2.0)*absig;
//			printf("%f %f %f\n",dtc->Max()-dtc->Min(),abmm,da1);
//			if ((dtc->Max()-dtc->Min())>da1*abmm || dtc->Sigma()<absig-da2) {
//			if (dtc->Sigma()>absig-da1 || dtc->Sigma()<absig-da2) {
			if (dtc->Sigma()<absig-da2) {
				delete dt;
				delete dtc;
				continue;
			}
			delete dtc;

			if (absig!=0) {
			// now we deal with the final threshold. This one checks to make
			// sure that the partical has reasonable azimuthal variations
			// the first threshold already dealt with radial variations
			RVEC rv(dt),*v2;
			bd=0;
			for (v2=abvec.first(); v2; v2=abvec.next()) bd=maxf(bd,v2->dot(rv));
			if (bd<thresh[3] && abvec.count()>1) {
				delete dt;
				continue;
			}
			}
*/
			// ok, we've got a good particle			
			k=newClip(i*2+x0-boxsize/2,j*2+y0-boxsize/2,boxsize,boxsize);
			if (data->hasCTFF()) {
				char st[BUF_SIZE],*t;
				string sc = string(data->Name());
				if(sc.size() > BUF_SIZE-24) {
					printf("The name %s is too long!!!", data->Name());
					exit(-1);
				}
				strcpy(st,data->Name());
				if ((t=strchr(st+3,' '))) *t=0;
				sprintf(st+strlen(st)," %d,%d",i*2+x0-boxsize/2,j*2+y0-boxsize/2);
				dt->setCTFF(st);
			}
			else {
				char st[BUF_SIZE];
				sprintf(st,"%d,%d : %55s",i*2+x0-boxsize/2,j*2+y0-boxsize/2,data->Path());
				dt->setName(st);
			}
			dt->normalize();
			ptcl.remove(k);
			ptcl.insert(k,dt);
			clipview->addData(dt);
			n++;
		}
	}
}
map->doneData();
clipview->setUpdatesEnabled(TRUE);
redisplayBoxes();
}



EMData *Control::ptclCorr(int xo,int yo,int smaller) { 
EMData *dc,*dd,*dn,*dn2,*ccf,*ref,*dt;
int i,j,ii,jj,k,x1,y1;
float *dp,*ddp,t,m,s;

// a little padding to get rid of edge effects
dd=data->clip(xo,yo,CLIP*2,CLIP*2);
dd->meanShrink(2);

#ifdef DEBUG0
printf("g0 mode\n");
dd->writeIMAGIC("yyy.hed",-1);
#endif


dc=dd->copy(0,0);
dn=dd->copy(0,0);
//dn2=dd->copy(0,0);
dd->zero();

//dn->realFilter(18);
//dn->filter(-5.,CLIP/50.0,1);

dn->filter(-5.,CLIP/50.0,1);
dc->subtract(dn);

#ifdef DEBUG1
dn->writeIMAGIC("yyy.hed",1);
dc->writeIMAGIC("yyy.hed",2);
#endif

if (!bestmap) bestmap=(unsigned char *)malloc(CLIP*CLIP);
else bestmap=(unsigned char *)realloc(bestmap,CLIP*CLIP);
for (i=0; i<abptcl.count(); i++) {
//	ptcl.at(i)->writeIMAGIC("ref.hed",-1);
	ref=abptcl.at(i)->clip(-(CLIP-boxsize/2)/2,-(CLIP-boxsize/2)/2,CLIP,CLIP);
//	ref->normalize();
	ref->multConst(1.0/ref->Sigma());
	ref->toCorner();
	ccf=dc->calcCCF(ref);
#ifdef DEBUG1
//	ccf->writeIMAGIC("yyy.hed",(i*2)+1);
//	ref->writeIMAGIC("yyy.hed",(i*2)+2);
#endif

	dp=ccf->getDataRO();
	ddp=dd->getData();
	for (j=0; j<CLIP*CLIP; j++) {
		if (dp[j]>ddp[j]) { ddp[j]=dp[j]; bestmap[j]=i; }
	}
	ccf->doneData();
	dd->doneData();
	
	delete ccf;
	delete ref;
}
#ifdef DEBUG1
dd->writeIMAGIC("yyy.hed",3);
#endif

// normalize the peak values
/*dp=dn->getDataRO();
ddp=dd->getData();
for (j=0; j<CLIP*CLIP; j++) {
	ddp[j]/=sqrt(dp[j]);
}
dd->doneData();
dn->doneData();*/

#ifdef DEBUG1
dn->writeIMAGIC("yyy.hed",4);
dd->writeIMAGIC("yyy.hed",5);
#endif

// This does a low pass filter so are peaks are guarenteed to be separated from each other
//dd->filter(0,CLIP/(boxsize*0.5554),5);	// This is a 'perfect' 1/2 box filter
//dd->filter(0,CLIP*8.0/(boxsize*.5554),5);
dd->filter(0,CLIP/6.0,5);
t=dd->Max();
//dd->writeIMAGIC("yyy.hed",i*2+2);
dp=dd->getData();

// first, we locate peaks
for(i=boxsize/4; i<CLIP-boxsize/4-1; i++) {
	for (j=boxsize/4; j<CLIP-boxsize/4-1; j++) {
		k=i+j*CLIP;
		if (dp[k]>dp[k-1] && dp[k]>dp[k+1] && dp[k]>dp[k+CLIP] && dp[k]>dp[k-CLIP] && dp[k]>0 &&
			dp[k]>dp[k-1-CLIP] && dp[k]>dp[k+1-CLIP] && dp[k]>dp[k-1+CLIP] && dp[k]>dp[k+1+CLIP]) {
				dp[k]+=t;
		}
	}
}

// Now we eliminate boxes which are too close to each other
for(i=boxsize/4; i<CLIP-boxsize/4-1; i++) {
	for (j=boxsize/4; j<CLIP-boxsize/4-1; j++) {
		k=i+j*CLIP;
		if (dp[k]>t) {
			x1=i+boxsize/4-1;
			y1=j+boxsize/4-1;
			for (ii=i-boxsize/4; ii<x1; ii++) {
				for (jj=j-boxsize/4; jj<y1; jj++) {
					if (dp[ii+jj*CLIP]>dp[k]) { dp[k]=0; break; }
				}
			}

		}
	}
}


/*
dd->doneData();
dd->writeMRC("tstc.mrc");
dp=dd->getData();
*/

// now we eliminate everything but the peaks
for (i=1; i<CLIP-1; i++) {
	for (j=1; j<CLIP-1; j++) {
		k=i+j*CLIP;
		if (dp[k]<=t) dp[k]=0; 
		else {
			dp[k]-=t;
			dp[k]*=100.0;
		}
	}
}

dd->doneData();

delete dc;
delete dn;
//delete dd;
return dd;
}

/*
EMData *Control::ptclCorr(int xo,int yo,int smaller) { 
EMData *dc,*dd,*ccf,*ref,*dt;
int i,j,ii,jj,k,x1,y1;
float *dp,*ddp,t,m,s;

// a little padding to get rid of edge effects
dd=data->clip(xo-128,yo-128,CLIP*2+256,CLIP*2+256);
dd->medianShrink(2);

// Now we fill in those padded zero regions with decent values
dp=dd->getDataRO();

dc=dd->clip(64,64,CLIP,6);
m=dc->Mean();
s=dc->Sigma();
delete dc;
for (j=0; j<64; j++) {
	for (i=0; i<CLIP+128; i++) {
		dp[i+j*(CLIP+128)]=grand(m,s);
	}
}

dc=dd->clip(64,64+CLIP-6,CLIP,6);
m=dc->Mean();
s=dc->Sigma();
delete dc;
for (j=CLIP+64; j<CLIP+128; j++) {
	for (i=0; i<CLIP+128; i++) {
		dp[i+j*(CLIP+128)]=grand(m,s);
	}
}

dc=dd->clip(64,64,6,CLIP);
m=dc->Mean();
s=dc->Sigma();
delete dc;
for (j=64; j<64+CLIP; j++) {
	for (i=0; i<64; i++) {
		dp[i+j*(CLIP+128)]=grand(m,s);
	}
}

dc=dd->clip(64+CLIP-6,64,6,CLIP);
m=dc->Mean();
s=dc->Sigma();
delete dc;
for (j=64; j<64+CLIP; j++) {
	for (i=CLIP+64; i<CLIP+128; i++) {
		dp[i+j*(CLIP+128)]=grand(m,s);
	}
}

dd->doneData();
dd->update();

#ifdef DEBUG0
dd->writeIMAGIC("xxx.hed",0);
#endif

dd->filter(3,10000,1);		// high pass filter to clean up gradients
#ifdef DEBUG0
dd->writeIMAGIC("xxx.hed",1);
#endif

dc=dd->clip(64,64,CLIP,CLIP);
delete dd;

dd=dc->copy(0,0);
dd->zero();

#ifdef DEBUG0
dc->writeIMAGIC("yyy.hed",0);
#endif

if (!bestmap) bestmap=(unsigned char *)malloc(CLIP*CLIP);
else bestmap=(unsigned char *)realloc(bestmap,CLIP*CLIP);
for (i=0; i<abptcl.count(); i++) {
//	ptcl.at(i)->writeIMAGIC("ref.hed",-1);
	ref=abptcl.at(i)->clip(-(CLIP-boxsize/2)/2,-(CLIP-boxsize/2)/2,CLIP,CLIP);
//	ref->normalize();
	ref->multConst(1.0/ref->Sigma());
	ref->toCorner();
	ccf=dc->calcCCF(ref);

#ifdef DEBUG0
	ccf->writeIMAGIC("yyy.hed",(i*2)+1);
	ref->writeIMAGIC("yyy.hed",(i*2)+2);
#endif

	dp=ccf->getDataRO();
	ddp=dd->getData();
	for (j=0; j<CLIP*CLIP; j++) {
		if (dp[j]>ddp[j]) { ddp[j]=dp[j]; bestmap[j]=i; }
	}
	ccf->doneData();
	dd->doneData();
	
	delete ccf;
	delete ref;
}

#ifdef DEBUG0
dd->writeIMAGIC("yyy.hed",i*2+1);
#endif

// This does a low pass filter so are peaks are guarenteed to be separated from each other
//dd->filter(0,CLIP/(boxsize*0.5554),5);	// This is a 'perfect' 1/2 box filter
//dd->filter(0,CLIP*8.0/(boxsize*.5554),5);
dd->filter(0,CLIP/6.0,5);
t=dd->Max();
//dd->writeIMAGIC("yyy.hed",i*2+2);
dp=dd->getData();

// first, we locate peaks
for(i=boxsize/4; i<CLIP-boxsize/4-1; i++) {
	for (j=boxsize/4; j<CLIP-boxsize/4-1; j++) {
		k=i+j*CLIP;
		if (dp[k]>dp[k-1] && dp[k]>dp[k+1] && dp[k]>dp[k+CLIP] && dp[k]>dp[k-CLIP] && dp[k]>0 &&
			dp[k]>dp[k-1-CLIP] && dp[k]>dp[k+1-CLIP] && dp[k]>dp[k-1+CLIP] && dp[k]>dp[k+1+CLIP]) {
				dp[k]+=t;
		}
	}
}

// Now we eliminate boxes which are too close to each other
for(i=boxsize/4; i<CLIP-boxsize/4-1; i++) {
	for (j=boxsize/4; j<CLIP-boxsize/4-1; j++) {
		k=i+j*CLIP;
		if (dp[k]>t) {
			x1=i+boxsize/4-1;
			y1=j+boxsize/4-1;
			for (ii=i-boxsize/4; ii<x1; ii++) {
				for (jj=j-boxsize/4; jj<y1; jj++) {
					if (dp[ii+jj*CLIP]>dp[k]) { dp[k]=0; break; }
				}
			}

		}
	}
}



// now we eliminate everything but the peaks
for (i=1; i<CLIP-1; i++) {
	for (j=1; j<CLIP-1; j++) {
		k=i+j*CLIP;
		if (dp[k]<=t) dp[k]=0; 
		else {
			dp[k]-=t;
			dp[k]*=100.0;
		}
	}
}

dd->doneData();

delete dc;
//delete dd;
return dd;
}
*/
EMData *Control::ptclCorrSVD(int xo,int yo,int smaller) { 
EMData *dc,*dd,*ccf,*ref,*dt;
int i,j,ii,jj,k,x1,y1;
float *dp,*ddp,t,m,s;

// a little padding to get rid of edge effects
dd=data->clip(xo,yo,CLIP*2,CLIP*2);
dd->medianShrink(2);

#ifdef DEBUG0
dd->writeIMAGIC("xxx.hed",0);
#endif

//dd->filter(3,10000,1);		// high pass filter to clean up gradients
#ifdef DEBUG0
dd->writeIMAGIC("xxx.hed",1);
#endif

dc=dd->copy(0,0);

dd->zero();

#ifdef DEBUG1
dc->writeIMAGIC("yyy.hed",0);
#endif

if (!bestmap) bestmap=(unsigned char *)malloc(CLIP*CLIP);
else bestmap=(unsigned char *)realloc(bestmap,CLIP*CLIP);
for (i=0; i<abptcl.count(); i++) {
//	ptcl.at(i)->writeIMAGIC("ref.hed",-1);
	ref=abptcl.at(i)->clip(-(CLIP-boxsize/2)/2,-(CLIP-boxsize/2)/2,CLIP,CLIP);
//	ref->normalize();
	ref->multConst(1.0/ref->Sigma());
	ref->toCorner();
	ccf=dc->calcCCF(ref);

#ifdef DEBUG1
	ccf->writeIMAGIC("yyy.hed",(i*2)+1);
	ref->writeIMAGIC("yyy.hed",(i*2)+2);
#endif

	dp=ccf->getDataRO();
	ddp=dd->getData();
	for (j=0; j<CLIP*CLIP; j++) ddp[j]+=dp[j]*dp[j]*abptcl.at(i)->NImg()/1000000.0;
	ccf->doneData();
	dd->doneData();
	
	delete ccf;
	delete ref;
}

#ifdef DEBUG1
dd->writeIMAGIC("yyy.hed",i*2+1);
#endif

t=dd->Max();
//dd->writeIMAGIC("yyy.hed",i*2+2);
dp=dd->getData();

// first, we locate peaks
for(i=boxsize/4; i<CLIP-boxsize/4-1; i++) {
	for (j=boxsize/4; j<CLIP-boxsize/4-1; j++) {
		k=i+j*CLIP;
		if (dp[k]>dp[k-1] && dp[k]>dp[k+1] && dp[k]>dp[k+CLIP] && dp[k]>dp[k-CLIP] && dp[k]>0 &&
			dp[k]>dp[k-1-CLIP] && dp[k]>dp[k+1-CLIP] && dp[k]>dp[k-1+CLIP] && dp[k]>dp[k+1+CLIP]) {
				dp[k]+=t;
		}
	}
}

// Now we eliminate boxes which are too close to each other
for(i=boxsize/4; i<CLIP-boxsize/4-1; i++) {
	for (j=boxsize/4; j<CLIP-boxsize/4-1; j++) {
		k=i+j*CLIP;
		if (dp[k]>t) {
			x1=i+boxsize/4-1;
			y1=j+boxsize/4-1;
			for (ii=i-boxsize/4; ii<x1; ii++) {
				for (jj=j-boxsize/4; jj<y1; jj++) {
					if (dp[ii+jj*CLIP]>dp[k]) { dp[k]=0; break; }
				}
			}

		}
	}
}


/*
dd->doneData();
dd->writeMRC("tstc.mrc");
dp=dd->getData();
*/

// now we eliminate everything but the peaks
for (i=1; i<CLIP-1; i++) {
	for (j=1; j<CLIP-1; j++) {
		k=i+j*CLIP;
		if (dp[k]<=t) dp[k]=0; 
		else {
			dp[k]-=t;
			dp[k]*=100.0;
		}
	}
}

dd->doneData();

delete dc;
//delete dd;
return dd;
}

void Control::Msvdfilt() {

}


/************************* START SNR SECTION *************************** 

  These functions are small modifications of the original functions,
  to account for the new selection criteria (using an estimate of SNR,
  for example).

  Mike Wakin -- August 2003

************************************************************************/

// This uses a set of reference projections for automatic boxing
void Control::autoBoxSNR() {
EMData *d1,*d2;
AutoboxSNR *ab;
int px,py,i;
ImShape *s;
List *refs;

QString f( QFileDialog::getOpenFileName() );
if ( f.isNull() ) return;
refs=readImages(f.data(),-1,-1);

QApplication::setOverrideCursor( waitCursor );
if (refs->count()<1) return;
if (refs->count()>250) error(ERR_EXIT,"FATAL ERROR: Max # references 250.");
Size->setText(QString::number(((EMData *)refs->objectAt(0))->xSize()));

Mnormalize();

abptcl.clear();
abvec.clear();
absig=0;
for (i=0; i<refs->count(); i++) {
	d1=(EMData *)refs->objectAt(i);
	d1->edgeNormalize();
	d2=d1->copy(0,0);
	d2->meanShrink(2); // originally medianShrink(2);
	d2->unum3=0;
	abptcl.append(d2);
	delete d1;
}
delete refs;


abmm=data->Max()-data->Min();

px=imview->getOriginx();
py=imview->getOriginy();
printf("px = %d, py = %d\n",px,py);
d1=ptclCorrSNR(px,py);

//d1->writeMRC("tstb.mrc");
ab=new AutoboxSNR();
printf("new complete\n");
ab->setup(px,py,d1->Min(),d1->Max(),d1,this);
printf("Setup complete\n");
ab->show();
QApplication::restoreOverrideCursor( );
}


void Control::autoBox2SNR(float *thresh) {
EMData *d1,*d2,*da;
int i,j,nx,ny;

nx=(data->xSize()-boxsize*2)/(CLIP*2-boxsize)+1;
ny=(data->ySize()-boxsize*2)/(CLIP*2-boxsize)+1;

delAll();

int SVD=((EMData *)abptcl.at(0))->unum3;

QProgressDialog progress( "Autoboxing", "Abort", nx*ny, this, 0 ,TRUE );
progress.setProgress( 0 );

for (i=0; i<ny; i++) {
	for (j=0; j<nx; j++) {
		progress.setProgress( i*nx+j );
		if (SVD) d1=ptclCorrSVD(j*(CLIP*2-boxsize),i*(CLIP*2-boxsize));
		else d1=ptclCorrSNR(j*(CLIP*2-boxsize),i*(CLIP*2-boxsize));
		boxesFromSNR(d1,j*(CLIP*2-boxsize),i*(CLIP*2-boxsize),0,0,thresh);
		delete d1;
		if ( progress.wasCancelled() ) return;
	}
}

}

void Control::boxesFromSNR(EMData *map,int x0,int y0,int sx,int sy,float *thresh) {
int i,j,k,l,c,n=0,mn;
float *d,*d2,da1,da2,bd;
EMData *dt,*dtc;

if (!map) return;
clipview->setUpdatesEnabled(FALSE);

//if (x0==0 && y0==0) delAll();

// sx and sy allows overlaping areas to be dealt with
if (sx<boxsize/4) sx=boxsize/4;
if (sy<boxsize/4) sy=boxsize/4;

mn=4*CLIP*CLIP*4/(boxsize*boxsize);

d=map->getData();
for (j=sy; j<CLIP-boxsize/4; j++) {
  for (i=sx; i<CLIP-boxsize/4; i++) {
    if (d[i+j*CLIP]>thresh[0]) {
      if (n>mn ||i*2+x0>data->xSize()-boxsize || j*2+y0>data->ySize()-boxsize) continue;
      dt=data->clip(i*2+x0-boxsize/2,j*2+y0-boxsize/2,boxsize,boxsize);
      
      // here we check for particles with large 'drawn' areas and eliminate them
      d2=dt->getDataRO();	
      c=0;
      for (l=0; l<boxsize*boxsize; l++) {
	if (d2[l]==0) c++;
      }
      dt->doneData();
      if (c>boxsize*2) {
	delete dt;
	continue;
      }			
      
      if (sqrt(resvars[i+j*CLIP])>thresh[1]) {
        //printf("Eliminating (VAR = %5.3f)\n",resvars[i+j*CLIP]);
        delete dt;
        continue;
      }
      if (specvars[i+j*CLIP]>thresh[2]) {
	//printf("Eliminating (WHITE = %5.3f)\n",specvars[i+j*CLIP]);
        delete dt;
	continue;
      }
      if (fabs(kurts[i+j*CLIP])>thresh[3]) {
	//printf("Eliminating (KURT = %5.3f)\n",kurts[i+j*CLIP]);
	delete dt;
	continue;
      }

      // ok, we've got a good particle			
      k=newClip(i*2+x0-boxsize/2,j*2+y0-boxsize/2,boxsize,boxsize);
      if (data->hasCTFF()) {
		char st[BUF_SIZE],*t;
		string sc = string(data->Name());
		if(sc.size() > BUF_SIZE-24) {
			printf("The name %s is too long!!!", data->Name());
			exit(-1);
		}
		strcpy(st,data->Name());
		if ((t=strchr(st+3,' '))) *t=0;
		sprintf(st+strlen(st)," %d,%d",i*2+x0-boxsize/2,j*2+y0-boxsize/2);
		dt->setCTFF(st);
      }
      else {
		char st[BUF_SIZE];
		sprintf(st,"%d,%d : %55s",i*2+x0-boxsize/2,j*2+y0-boxsize/2,data->Path());
		dt->setName(st);
	      }
	      dt->normalize();
	      ptcl.remove(k);
	      ptcl.insert(k,dt);
	      clipview->addData(dt);
	      n++;
      }
  }
}
map->doneData();
clipview->setUpdatesEnabled(TRUE);
redisplayBoxes();
}

EMData *Control::ptclCorrSNR(int xo,int yo,int smaller) { 
EMData *dc,*dd,*dn,*ccf,*rf,*orig,*thisbox,*tempbox,*res,*bestres,*brfft;
int i,j,ii,jj,k,x1,y1,xx,yy,xmax,ymax,cy,refindex,bestref;
float *dp,*ddp,t,mean0,var0,mean1,amp1,var1,bestamp,minvar1,tot,mean;
float *tempdata,*thisdata;
float numpixels = SQR(boxsize/2);

int SNRDEBUG = 0;

// This function will identify candidate particles, estimate the SNR on
// each candidate box, and compute some quantities to be compared against
// the thresholds.

// From this point forward, snrboxer.C and ptclCorrSNR in Control.cpp
// should be identical, except for areas marked by BATCH.

if (!bestmap) bestmap=(unsigned char *)malloc(CLIP*CLIP);
else bestmap=(unsigned char *)realloc(bestmap,CLIP*CLIP);

if (!resvars) resvars=(float *)malloc(CLIP*CLIP*sizeof(float));
else resvars=(float *)realloc(resvars,CLIP*CLIP*sizeof(float));

if (!kurts) kurts=(float *)malloc(CLIP*CLIP*sizeof(float));
else kurts=(float *)realloc(kurts,CLIP*CLIP*sizeof(float));

if (!specvars) specvars=(float *)malloc(CLIP*CLIP*sizeof(float));
else specvars=(float *)realloc(specvars,CLIP*CLIP*sizeof(float));

// a little padding to get rid of edge effects
dd=data->clip(xo,yo,CLIP*2,CLIP*2);

dd->meanShrink(2);

dc=dd->copy(0,0);
dn=dd->copy(0,0);
orig=dd->copy(0,0);
dd->zero();

// First attempt to account for background drift
dn->filter(-5,CLIP/50.0,1);
dc->subtract(dn);

for (i=0; i<abptcl.count(); i++) {
  // BATCH
    //rf = ((EMData *)abptcl.objectAt(i))->copy(0,0);
    //rf = rf->clip(-(CLIP-boxsize/2)/2,-(CLIP-boxsize/2)/2,CLIP,CLIP);
  // NON-BATCH
    rf = abptcl.at(i)->copy(0,0);
    rf = rf->clip(-(CLIP-boxsize/2)/2,-(CLIP-boxsize/2)/2,CLIP,CLIP);

  rf->multConst(1.0/rf->Sigma());

  rf->doneData();
  rf->update();

  ccf=dc->calcCCF(rf,1);
  
  dp=ccf->getDataRO();
  ddp=dd->getData();
  for (j=0; j<CLIP*CLIP; j++) {
    if (dp[j]>ddp[j]) { ddp[j]=dp[j]; bestmap[j]=i; }
  }
  ccf->doneData();
  dd->doneData();
  rf ->doneData();
  
  delete rf;
  delete ccf;
}

// low pass filter so peaks are separated from each other
dd->filter(0,CLIP/6.0,5);
dd->doneData();
dd->update();
t=dd->Max();

dp=dd->getData();
 
//locate peaks
for(i=boxsize/4; i<CLIP-boxsize/4-1; i++) {
  for (j=boxsize/4; j<CLIP-boxsize/4-1; j++) {
    k=i+j*CLIP;
    if (dp[k]>dp[k-1] && dp[k]>dp[k+1] && dp[k]>dp[k+CLIP] && dp[k]>dp[k-CLIP] && dp[k]>0 &&
	dp[k]>dp[k-1-CLIP] && dp[k]>dp[k+1-CLIP] && dp[k]>dp[k-1+CLIP] && dp[k]>dp[k+1+CLIP]) {
      dp[k]+=t;
    }
  }
}

// eliminate boxes which are too close to each other
for(i=boxsize/4; i<CLIP-boxsize/4-1; i++) {
  for (j=boxsize/4; j<CLIP-boxsize/4-1; j++) {
    k=i+j*CLIP;
    if (dp[k]>t) {
      x1=i+boxsize/4-1;
      y1=j+boxsize/4-1;
      for (ii=i-boxsize/4; ii<x1; ii++) {
	for (jj=j-boxsize/4; jj<y1; jj++) {
	  if (dp[ii+jj*CLIP]>dp[k]) { dp[k]=0; break; }
	}
      }    
    }
  }
}

// eliminate everything but the peaks
for (i=1; i<CLIP-1; i++) {
  for (j=1; j<CLIP-1; j++) {
    k=i+j*CLIP;
    if (dp[k]<=t) dp[k]=0; 
    else {
      dp[k]-=t;
      dp[k]*=100.0;
    }
  }
}

dd->doneData();
dp=dd->getData();
 
// Estimate SNR for candidate particles
for(i=boxsize/4; i<CLIP-boxsize/4-1; i++) {
  for (j=boxsize/4; j<CLIP-boxsize/4-1; j++) {
    k=i+j*CLIP;
    if (dp[k]>0) { 
      // Get this box
      thisbox = orig->clip(i-boxsize/4,j-boxsize/4,boxsize/2,boxsize/2);
      
      if ((thisbox->Max() == 0) && (thisbox->Min() == 0)) {
	// No data here -- do nothing.
	dp[k] = -1;
	bestmap[k] = 0;
	resvars[k] = 0;
	kurts[k] = 0;
	specvars[k] = 0;
      } else {
	if (SNRDEBUG) printf("Got thisbox i=%d, j=%d... ",i,j);

	// Suppose there is only noise (Hypothesis 0). Estimate mean
	// and variance.
	mean0 = thisbox->Mean();
	var0 = SQR(thisbox->Sigma());
	if (SNRDEBUG) printf("mean0 = %4.3f, var0 = %4.3f, ",mean0,var0);
	
	// Suppose there is particle + noise (Hypothesis 1).  For each
	// reference, find the corresponding amplitude, mean and
	// variance. Keep the reference that minimizes residual noise
	// energy.
	minvar1 = 1e20;
	for (refindex=0; refindex<abptcl.count(); refindex++) {
	  // BATCH 
	    // rf = ((EMData *)abptcl.objectAt(refindex))->copy(0,0);
	  // NON-BATCH
	    rf = abptcl.at(refindex)->copy(0,0);	
	  	
	  // Estimate mean and amplitude; compute variance of residual   
          tempbox = rf->copy(0,0);
          tempdata=tempbox->getDataRO();
          thisdata=thisbox->getDataRO();
	  // Careful with last argument to lsqfit (when 1, excludes
	  // zeros in fitting).
          lsqfit(numpixels,tempdata,thisdata,&amp1,&mean1,1);
          tempbox->doneData();
          thisbox->doneData();
          delete tempbox;
          tempbox = rf->copy(0,0);
          tempbox->multConst(-amp1);
          tempbox->addConst(-mean1);
          tempbox->add(thisbox);
          res = tempbox->copy(0,0);
          var1 = SQR(tempbox->Sigma());

	  if (var1 < minvar1) {
	    minvar1 = var1;
	    bestref = refindex;
	    bestres = res->copy(0,0);
	    bestamp = amp1;
	  }
	  
	  delete tempbox;
	  rf->doneData();
	  delete rf;
	  delete res;
	}

	if (bestamp > 0) {
	  // Let's only consider particles that seem to be positive

	  // Now, estimate SNR. This quantity should be invariant to
	  // scales and mean shifts of the observations.  (Note: the
	  // proper criteria to use in distinguishing white noise from
	  // particle would be dp[k] = var0/minvar1, but this is very
	  // similar.) We divide by 4.0 to account for the downsampling.
	  dp[k] = SQR(bestamp)/minvar1/4.0; 
	  resvars[k] = minvar1;
	  kurts[k] = bestres->Kurtosis();
	  bestmap[k] = bestref;
	    
	  // Want to analyze the residual image to eliminate junk.
	  // We normalize by var0 so that the residual image is 
	  // invariant to scales and mean shifts of the observations.
	  // This is not the same as normalizing by the variance of the 
	  // residual -- some will have more energy than others.
	  bestres->multConst(1.0/sqrt(var0)); 

	  // We would like the residual to be noise only (no junk).
	  // We've already accounted for noise energy with the SNR
	  // criteria; we need to check whether the block contains
	  // junk. For this criteria, we look at the FFT of the
	  // residual at low frequencies. There should only be small
	  // variation of energy at these frequencies. We compute the
	  // variation and call it the spectral variation of the
	  // residual. The lower the value, the more the residual
	  // resembles white noise. (Typically, pure white noise will have
	  // a value of 2 or lower.)
	  brfft = bestres->doFFT();
	  bestres->gimmeFFT();
	  brfft->ri2ap();
	  ymax = 4;
	  xmax = ymax*2;
	  cy = boxsize/4;
	  tot = 0;
	  for (xx=0;xx<xmax;xx+=2) {
	    for (yy=(cy-ymax);yy<=(cy+ymax);yy++) {
	      tot += brfft->valueAt(xx,yy);
	    }
	  }
	  mean = tot/(2.0*ymax+1.0)/(xmax/2.0);
	  tot = 0; 
	  for (xx=0;xx<xmax;xx+=2) {
	    for (yy=(cy-ymax);yy<=(cy+ymax);yy++) {
	      tot += SQR(brfft->valueAt(xx,yy)-mean);
	    }
	  }
	  specvars[k] = 10000*tot/(2.0*ymax+1.0)/(xmax/2.0);

	  if (SNRDEBUG) printf("minvar1 = %4.3f, bestamp = %4.3f, refindex = %d, SNR = %4.3f, specvars = %4.3f\n",minvar1,bestamp,bestmap[k],dp[k],specvars[k]);

	  delete brfft;

	} else {
	  // bestamp was negative -- not a likely match
	  dp[k] = -1;
	  bestmap[k] = 0;
	  resvars[k] = 0;
	  kurts[k] = 0;
	  specvars[k] = 0;

	  if (SNRDEBUG) printf("NEG AMP.\n");
	}

	delete thisbox;
	delete bestres;

      }    
    }
  }
}

dd->doneData();

delete dc; 
delete dn;
delete orig;
return dd; 
}

/************************* END SNR SECTION ******************************/ 



/*
#define CLIP 512		// 1/2 clip size


void Control::autoBox2() { 
EMData *d1,*d2,*d3,*d4,*da,*dc;
float *y,*y2,*y3,*y4,m,mf2,r,a,rm;
int nx,i,j,mf;

// calculate average fft of 'background'
d1=ptcl.first();
nx=d1->xSize();
d1->edgeNormalize();
d1->applyMask(-1,4);
d1->normalize();
da=d1->doFFT()->copy();

while (d1=ptcl.next()) {
	d1->edgeNormalize();
	d1->applyMask(-1,4);
	d1->normalize();
	d2=d1->doFFT();
	da->addIncoherent(d2);
}

delAll();

y=malloc(26*sizeof(float));
y2=malloc(26*sizeof(float));
abft->calcRadDist(25,3.0,(nx-3.0)/35.0,y);
da->calcRadDist(25,3.0,(nx-3.0)/35.0,y2);

//abft->writeMRC("tsta.mrc");
//da->writeMRC("tstb.mrc");

delete da;
delete abft;
abft=NULL;

// find the frequency with the most information
mf=0;
m=-1.0e20;
for (i=0; i<25; i++) {
	if (y[i]-y2[i]>m) { m=y[i]-y2[i]; mf=i; }
}

//save_data(3.0,(nx-3.0)/35.0, y, 25, "t1");
//save_data(3.0,(nx-3.0)/35.0, y2, 25, "t2");

free(y);
free(y2);

mf2=(mf+3)*(nx-3.0)/35.0;
printf("Max freq=%d %f %f pix\n",mf,mf2,nx/mf2);
mf2=nx/mf2;

dc=data->clip(0,0,1024,1024);

da=new EMData;
da->setSize(CLIP*2,CLIP*2,1);
rm=SQR(.85*nx/2.0);

// generate cos*cos fn and CCF
da->zero();
y=da->getData();
for (i=CLIP-nx/2; i<CLIP+nx/2; i++) {
	for (j=CLIP-nx/2; j<CLIP+nx/2; j++) {
		r=SQR(i-CLIP)+SQR(j-CLIP);
		y[i+j*CLIP*2]=cos(i*2.0*PI/mf2)*cos(j*2.0*PI/mf2)*(r>rm?-1.0:1.0)*exp(-25.0*r/(nx*nx));
	}
}
da->doneData();
da->writeMRC("tst1.mrc");
da->toCorner();
d1=dc->calcCCF(da);

// generate sin*sin fn and CCF
da->zero();
y=da->getData();
for (i=CLIP-nx/2; i<CLIP+nx/2; i++) {
	for (j=CLIP-nx/2; j<CLIP+nx/2; j++) {
		r=SQR(i-CLIP)+SQR(j-CLIP);
		y[i+j*CLIP*2]=sin(i*2.0*PI/mf2)*sin(j*2.0*PI/mf2)*(r>rm?-1.0:1.0)*exp(-25.0*r/(nx*nx));
	}
}
da->doneData();
da->writeMRC("tst2.mrc");
da->toCorner();
d2=dc->calcCCF(da);

// generate sin*cos fn and CCF
da->zero();
y=da->getData();
for (i=CLIP-nx/2; i<CLIP+nx/2; i++) {
	for (j=CLIP-nx/2; j<CLIP+nx/2; j++) {
		r=SQR(i-CLIP)+SQR(j-CLIP);
		y[i+j*CLIP*2]=sin(i*2.0*PI/mf2)*cos(j*2.0*PI/mf2)*(r>rm?-1.0:1.0)*exp(-25.0*r/(nx*nx));
	}
}
da->doneData();
da->writeMRC("tst3.mrc");
da->toCorner();
d3=dc->calcCCF(da);

// generate cos*sin fn and CCF
da->zero();
y=da->getData();
for (i=CLIP-nx/2; i<CLIP+nx/2; i++) {
	for (j=CLIP-nx/2; j<CLIP+nx/2; j++) {
		r=SQR(i-CLIP)+SQR(j-CLIP);
		y[i+j*CLIP*2]=cos(i*2.0*PI/mf2)*sin(j*2.0*PI/mf2)*(r>rm?-1.0:1.0)*exp(-25.0*r/(nx*nx));
	}
}
da->doneData();
da->writeMRC("tst4.mrc");
da->toCorner();
d4=dc->calcCCF(da);


d1->ri2ap();
d2->ri2ap();

y=d1->getData();
y2=d2->getDataRO();
y3=d3->getDataRO();
y4=d4->getDataRO();
for (i=0; i<1024*1024; i++) {
	y[i]=sqrt(y[i]*y[i]+y2[i]*y2[i]+y3[i]*y3[i]+y4[i]*y4[i]);
}
d1->doneData();
d2->doneData();
d3->doneData();
d4->doneData();

d1->writeMRC("tsta.mrc");
dc->writeMRC("tstb.mrc");

delete d1;
delete d2;
delete d3;
delete d4;
delete dc;
delete da;
}

*/

