#include "emobject.h"

EMObject::operator int () const
{
	if (type == INT) {
		return n;
	}
	else if (type == FLOAT) {
		return (int) f;
	}
	else if (type == DOUBLE) {
		return (int) d;
	}
	else {
		if (type != UNKNOWN) {
			fprintf(stderr, "Cannot convert to int this data type\n");
		}
	}
	return 0;
}

EMObject::operator float () const
{
	if (type == FLOAT) {
		return f;
	}
	else if (type == INT) {
		return (float) n;
	}
	else if (type == DOUBLE) {
		return (float) d;
	}
	else {
		if (type != UNKNOWN) {
			fprintf(stderr, "Cannot convert to float from this data type\n");
		}
	}

	return 0;
}

EMObject::operator double () const
{
	if (type == DOUBLE) {
		return d;
	}
	else if (type == INT) {
		return (double) n;
	}
	else if (type == FLOAT) {
		return (double) f;
	}
	else {
		if (type != UNKNOWN) {
			fprintf(stderr, "Cannot convert to double from this data type\n");
		}
	}
	return 0;
}

EMObject::operator  const char *() const
{
	if (type != STRING) {
		if (type != UNKNOWN) {
			fprintf(stderr, "Cannot convert to string from this data type\n");
		}
		return "";
	}
	return str.c_str();
}

EMObject::operator EMData * () const
{
	if (type != EMDATA) {
		if (type != UNKNOWN) {
			fprintf(stderr, "Cannot convert to EMData* from this data type\n");
		}
		return 0;
	}
	return emdata;
}

EMObject::operator  XYData * () const
{
	if (type != XYDATA) {
		if (type != UNKNOWN) {
			fprintf(stderr, "Cannot convert to XYData* from this data type\n");
		}
		return 0;
	}
	return xydata;
}

EMObject::operator vector < float > () const
{
	if (type != FLOATARRAY) {
		if (type != UNKNOWN) {
			fprintf(stderr, "Cannot convert to vector<float> from this data type\n");
		}
		return vector < float >();
	}
	return farray;
}

EMObject::operator vector < int > () const
{
	if (type != INTARRAY) {
		if (type != UNKNOWN) {
			fprintf(stderr, "Cannot convert to vector<int> from this data type\n");
		}
		return vector < int >();
	}
	return iarray;
}

EMObject::operator vector<string> () const
{
	if (type != STRINGARRAY) {
		if (type != UNKNOWN) {
			fprintf(stderr, "Cannot convert to vector<string> from this data type\n");
		}
		return vector<string>();
	}
	return strarray;
}

bool EMObject::is_null() const
{
	return (type == UNKNOWN);
}

string EMObject::to_str() const
{
	if (type == STRING) {
		return str;
	}
	else {
		char tmp_str[32];

		if (type == INT) {
			sprintf(tmp_str, "%d", n);
		}
		else if (type == FLOAT) {
			sprintf(tmp_str, "%f", f);
		}
		else if (type == DOUBLE) {
			sprintf(tmp_str, "%f", d);
		}
		else if (type == EMDATA) {
			sprintf(tmp_str, "EMDATA");
		}
		else if (type == XYDATA) {
			sprintf(tmp_str, "XYDATA");
		}
		else {
			fprintf(stderr, "No such EMObject defined\n");
		}
		return string(tmp_str);
	}
}

EMObject::ObjectType EMObject::get_type() const
{
	return type;
}


const char *EMObject::get_object_type_name(ObjectType t)
{
	switch (t) {
	case INT:
		return "INT";
	case FLOAT:
		return "FLOAT";
	case DOUBLE:
		return "DOUBLE";
	case STRING:
		return "STRING";
	case EMDATA:
		return "EMDATA";
	case XYDATA:
		return "XYDATA";
	case FLOATARRAY:
		return "FLOATARRAY";
	case INTARRAY:
		return "INTARRAY";
	case STRINGARRAY:
		return "STRINGARRAY";
	case UNKNOWN:
		fprintf(stderr, "No such EMObject defined\n");
	}

	return "UNKNOWN";
}


bool operator==(const EMObject &e1, const EMObject & e2)
{
#if 0
	if (e1.type != e2.type) {
		return false;
	}
#endif
	switch (e1.type) {
	case EMObject::INT:
		return (e1.n == e2.n);
	case EMObject::FLOAT:
		return (e1.f == e2.f);
	case EMObject::DOUBLE:
		return (e1.d == e2.d);
	case EMObject::STRING:
		return (e1.str == e2.str);
	case EMObject::EMDATA:
		return (e1.emdata == e2.emdata);
	case EMObject::XYDATA:
		return (e1.xydata == e2.xydata);
	case EMObject::FLOATARRAY:
		if (e1.farray.size() == e2.farray.size()) {
			for (size_t i = 0; i < e1.farray.size(); i++) {
				if (e1.farray[i] != e2.farray[i]) {
					return false;
				}
			}
			return true;
		}
		else {
			return false;
		}
	case EMObject::INTARRAY:
		if (e1.iarray.size() == e2.iarray.size()) {
			for (size_t i = 0; i < e1.iarray.size(); i++) {
				if (e1.iarray[i] != e2.iarray[i]) {
					return false;
				}
			}
			return true;
		}
		else {
			return false;
		}
	case EMObject::STRINGARRAY:
		if (e1.strarray.size() == e2.strarray.size()) {
			for (size_t i = 0; i < e1.strarray.size(); i++) {
				if (e1.strarray[i] != e2.strarray[i]) {
					return false;
				}
			}
			return true;
		}
		else {
			return false;
		}
	default:
		return false;
	}
	return false;
}

bool operator!=(const EMObject &e1, const EMObject & e2)
{
	return !(e1 == e2);
}
