#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "util.h"
#include <iostream>
#include <stdlib.h>

EMData * getPtcl(EMData * src, int x0, int y0, int x1, int y1, int r, int length=0, char * name=NULL)
{
    float x2=x1-x0;
    float y2=y1-y0;
    float ln=hypot(x2,y2);
    float px= y2/ln;
    float py=-x2/ln;
	x2/=ln;
	y2/=ln;
	ln=floor(ln);
	if (length==0) length=(int)ln;
	EMData *sv = new EMData;
	sv->setSize(r*2,length);
	sv->getData();
	float x,y,v;
	float p1, p2, p3, p4;
	int xx, yy;
	float mean=src->Mean();
	for (y=0; y<length; y+=1.0) {
		for (x=-r; x<r; x+=1.0) {
			float v=src->valueAtInterp(x0+x*px+y*x2,y0+x*py+y*y2);
/*			xx=(int)floor(x0+x*px+y*x2);
			yy=(int)floor(y0+x*py+y*y2);
			p1=src->valueAtSafe(xx,yy);
			p2=src->valueAtSafe(xx+1,yy);
			p3=src->valueAtSafe(xx+1,yy+1);
			p4=src->valueAtSafe(xx,yy+1);
			if ( p1==0.0 || p2==0.0 || p3==0.0 || p4==0.0)
				v=mean;
			else
				v=bilin(p1,p2,p3,p4,x0+x*px+y*x2-xx,y0+x*py+y*y2-yy);*/
			sv->setValueAt((int)x+r,(int)y,0,v);
		}
	}
	sv->doneData();
	sv->setName(name);
	return sv;
	
}


EMData * getPtcl2(EMData * src, int x0, int y0, int x1, int y1, int r, int start=0, int length=0, char * name=NULL)
{
    float x2=x1-x0;
    float y2=y1-y0;
    float ln=hypot(x2,y2);
    float px= y2/ln;
    float py=-x2/ln;
	x2/=ln;
	y2/=ln;
	ln=floor(ln);
	if (length==0) length=(int)ln;
	EMData *sv = new EMData;
	sv->setSize(r*2,length-start);
	sv->getData();
	float x,y,v;
	float p1, p2, p3, p4;
	int xx, yy;
	float mean=src->Mean();
	for (y=start; y<length; y+=1.0) {
		for (x=-r; x<r; x+=1.0) {
			float v=src->valueAtInterp(x0+x*px+y*x2,y0+x*py+y*y2);
/*			xx=(int)floor(x0+x*px+y*x2);
			yy=(int)floor(y0+x*py+y*y2);
			p1=src->valueAtSafe(xx,yy);
			p2=src->valueAtSafe(xx+1,yy);
			p3=src->valueAtSafe(xx+1,yy+1);
			p4=src->valueAtSafe(xx,yy+1);
			if ( p1==0.0 || p2==0.0 || p3==0.0 || p4==0.0)
				v=mean;
			else
				v=bilin(p1,p2,p3,p4,x0+x*px+y*x2-xx,y0+x*py+y*y2-yy);*/
			sv->setValueAt((int)x+r,(int)y-start,0,v);
		}
	}
	sv->doneData();
	sv->setName(name);
	return sv;
	
}


EMData * getPtclNoInterp2(EMData * src, int x0, int y0, int x1, int y1, int r, int start=0, int length=0, int mask=-1, char * name=NULL)
{
	float x2=x1-x0;
	float y2=y1-y0;
	float midpoint=(length+start)/2.;
	float ln=hypot(x2,y2);
	float x3=x0+(midpoint/ln)*x2;
	float y3=y0+(midpoint/ln)*y2;
	int ix3=(int) floor2(x3);
	int iy3=(int) floor2(y3);
	EMData * sv=new EMData;
	sv->setSize(r*2,r*2);
	float *data=sv->getData();
	float *datasrc=src->getDataRO();
	for (int j=iy3-r; j<iy3+r; j++)
		for (int i=ix3-r; i<ix3+r; i++)
			data[i-ix3+r+sv->xSize()*(j-iy3+r)]=datasrc[i+j*src->xSize()];
	sv->doneData();
	src->doneData();
	if (mask>=0) sv->applyMask(mask,0,0,0,0);
	return sv;
}

EMData * getPtclNoInterp(EMData * src, int x0, int y0, int x1, int y1, int r, int length=0, int mask=-1, char * name=NULL)
{
	return getPtclNoInterp2(src, x0, y0, x1, y1, r, 0, length, mask, name);
}

main(int argc,char *argv[]) {
	char * boxed, * input, * dbin;
	int length=0,width=0,seg=0,maskwidth=0,i;
	float segoverlap=0.1;
	char t[121];
	char name[200];
	bool nointerp=false, nomask=false;
	
	appinit(argc,argv);
	if (argc<3) appusage(argv[0]);
	int Ref;
	Ref=LOGbegin(argc, argv);
	for (i=1; i<argc; i++) {	
		if (strncmp(argv[i],"input=",6)==0) input=argv[i]+6;
		else if (strncmp(argv[i],"output=",7)==0) boxed=argv[i]+7; 
		else if (strncmp(argv[i],"length=",7)==0) length=atoi(argv[i]+7); 
		else if (strncmp(argv[i],"width=",6)==0) width=atoi(argv[i]+6); 
		else if (strncmp(argv[i],"mask=",5)==0) maskwidth=atoi(argv[i]+5); 
		else if (strncmp(argv[i],"seg=",4)==0) seg=atoi(argv[i]+4); 
		else if (strncmp(argv[i],"overlap=",8)==0) segoverlap=atof(argv[i]+8)/100.0; 
		else if (strncmp(argv[i],"dbbox=",6)==0) dbin=argv[i]+6;
		else if (strncmp(argv[i],"nointerp",8)==0) nointerp=true;
		else if (strncmp(argv[i],"nomask",6)==0) nomask=true;
		
		else { 
			printf("Unknown option %s\n",argv[i]);
			exit(1); 	
		}
	}
	if (segoverlap!=0.1) printf("Using overlap: %6.2f %, %d pixels\n",segoverlap*100.0,int(seg*segoverlap));
	if (!input) error(ERR_EXIT,"Cannot output boxed particles without an input image\n");
	LOG(Ref, input, LOG_INFILE, NULL);
	if (seg!=0 && length!=0) error(ERR_EXIT, "Cannot specify seg and length together\n");
	if (nointerp && maskwidth==0) error(ERR_EXIT, "Please specify width of true particle only (not width of box)\n");
	EMData image;
	image.readImage(input,0);
	FILE *in=fopen((const char *)dbin,"r");
	while (fgets(t,120,in)) {
	    int a0,a1,a2,a3,a4=0,a5,a6;
	    int x0, y0, x1, y1, r;
	    float x2, y2, px, py;
	    sscanf(t," %d %d %d %d %d",&a0,&a1,&a2,&a3,&a4);
	    if (a4==-1) {
			fgets(t,120,in);
			sscanf(t," %d %d %d %d %d",&a2,&a3,&a4,&a5,&a6);
			if (a6!=-2) continue;
			x0=a0+a4/2;
			y0=a1+a4/2;
			x1=a2+a4/2;
			y1=a3+a4/2;
			if (width==0) r=a4/2;
			else r=width/2;
			if (length>0) {
				float xc = (x0 + x1) / 2.0;
				float yc = (y0 + y1) / 2.0;
				float x2=x1-x0;
			    float y2=y1-y0;
			    float ln=hypot(x2,y2);
				x2/=ln;
				y2/=ln;
				int half=length/2;
				x1=(int) floor(xc+half*x2);
				y1=(int) floor(yc+half*y2);
				x0=(int) floor(xc-half*x2);
				y0=(int) floor(yc-half*y2);
			}
			if (seg>0) {
				float x2=x1-x0;
			    	float y2=y1-y0;
			    	float ln=hypot(x2,y2);
				for (i=0; i<ln-seg+1; i+=seg*segoverlap)
				{
					EMData * sv;
					sprintf(name,"File: %s|%d\t%d\t%d\t%d\t-1|%d\t%d\t%d\t%d\t-2",
						input,int(floor2(x0-r+0.5)),int(floor2(y0-r+0.5)),int(floor2(r*2+0.5)),int(floor2(r*2+0.5)),int(floor2(x1-r+0.5)),int(floor2(y1-r+0.5)),int(floor2(r*2+0.5)),int(floor2(r*2+0.5)));
					if (nointerp) {
						int mask=(int)floor2(hypot(seg/2.,maskwidth))+1;
						if (nomask) mask=-1;
						sv=getPtclNoInterp2(&image, x0, y0, x1, y1, r, i, i+seg, mask, name);
					}
					else 
						sv=getPtcl2(&image, x0, y0, x1, y1, r, i, i+seg, name);
					sv->writeImage(boxed, -1);
					delete sv;	
				}
			}
			else
			{	
			EMData * sv;
			sprintf(name,"File: %s|%d\t%d\t%d\t%d\t-1|%d\t%d\t%d\t%d\t-2",
				input,int(floor2(x0-r+0.5)),int(floor2(y0-r+0.5)),int(floor2(r*2+0.5)),int(floor2(r*2+0.5)),int(floor2(x1-r+0.5)),int(floor2(y1-r+0.5)),int(floor2(r*2+0.5)),int(floor2(r*2+0.5)));
			if (nointerp) {
				int mask=(int) hypot(length,width);
				if (nomask) mask=-1;
				sv=getPtclNoInterp(&image, x0, y0, x1, y1, r, length, mask, name);
			}
			else
				sv=getPtcl(&image, x0, y0, x1, y1, r, length, name);
				
			sv->writeImage(boxed, -1);
			delete sv;	
			}
	    }	
	}
	fclose(in);
	LOG(Ref, boxed, LOG_OUTFILE, NULL);
}
